@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "bewaveddolphin.h"
@@ -10,11 +10,14 @@
 #include "globalproperties.h"
 #include "graphicelement.h"
 #include "graphicelementinput.h"
+#include "inputrotary.h"
 #include "lengthdialog.h"
 #include "mainwindow.h"
+#include "serialization.h"
 #include "settings.h"
 #include "simulationblocker.h"
 
+#include <QAbstractItemView>
 #include <QClipboard>
 #include <QCloseEvent>
 #include <QFileDialog>
@@ -35,13 +38,8 @@ SignalModel::SignalModel(const int inputs, const int rows, const int columns, QO
 
 Qt::ItemFlags SignalModel::flags(const QModelIndex &index) const
 {
-    Qt::ItemFlags flags = Qt::ItemIsSelectable | Qt::ItemIsEnabled;
-
-    if (index.row() < m_inputCount) {
-        flags |= Qt::ItemIsEditable;
-    }
-
-    return flags;
+    Q_UNUSED(index)
+    return Qt::ItemIsSelectable | Qt::ItemIsEnabled;
 }
 
 SignalDelegate::SignalDelegate(QObject *parent)
@@ -81,36 +79,39 @@ BewavedDolphin::BewavedDolphin(Scene *scene, const bool askConnection, MainWindo
     m_view.setScene(m_scene);
     m_view.setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     m_view.setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    m_view.setRedirectZoom(true);
     m_ui->verticalLayout->addWidget(&m_view);
 
     m_ui->mainToolBar->setToolButtonStyle(Settings::value("labelsUnderIcons").toBool() ? Qt::ToolButtonTextUnderIcon : Qt::ToolButtonIconOnly);
 
     loadPixmaps();
 
-    connect(m_ui->actionAbout,         &QAction::triggered, this, &BewavedDolphin::on_actionAbout_triggered);
-    connect(m_ui->actionAboutQt,       &QAction::triggered, this, &BewavedDolphin::on_actionAboutQt_triggered);
-    connect(m_ui->actionClear,         &QAction::triggered, this, &BewavedDolphin::on_actionClear_triggered);
-    connect(m_ui->actionCombinational, &QAction::triggered, this, &BewavedDolphin::on_actionCombinational_triggered);
-    connect(m_ui->actionCopy,          &QAction::triggered, this, &BewavedDolphin::on_actionCopy_triggered);
-    connect(m_ui->actionCut,           &QAction::triggered, this, &BewavedDolphin::on_actionCut_triggered);
-    connect(m_ui->actionExit,          &QAction::triggered, this, &BewavedDolphin::on_actionExit_triggered);
-    connect(m_ui->actionExportToPdf,   &QAction::triggered, this, &BewavedDolphin::on_actionExportToPdf_triggered);
-    connect(m_ui->actionExportToPng,   &QAction::triggered, this, &BewavedDolphin::on_actionExportToPng_triggered);
-    connect(m_ui->actionFitScreen,     &QAction::triggered, this, &BewavedDolphin::on_actionFitScreen_triggered);
-    connect(m_ui->actionInvert,        &QAction::triggered, this, &BewavedDolphin::on_actionInvert_triggered);
-    connect(m_ui->actionLoad,          &QAction::triggered, this, &BewavedDolphin::on_actionLoad_triggered);
-    connect(m_ui->actionPaste,         &QAction::triggered, this, &BewavedDolphin::on_actionPaste_triggered);
-    connect(m_ui->actionResetZoom,     &QAction::triggered, this, &BewavedDolphin::on_actionResetZoom_triggered);
-    connect(m_ui->actionSave,          &QAction::triggered, this, &BewavedDolphin::on_actionSave_triggered);
-    connect(m_ui->actionSaveAs,        &QAction::triggered, this, &BewavedDolphin::on_actionSaveAs_triggered);
-    connect(m_ui->actionSetClockWave,  &QAction::triggered, this, &BewavedDolphin::on_actionSetClockWave_triggered);
-    connect(m_ui->actionSetLength,     &QAction::triggered, this, &BewavedDolphin::on_actionSetLength_triggered);
-    connect(m_ui->actionSetTo0,        &QAction::triggered, this, &BewavedDolphin::on_actionSetTo0_triggered);
-    connect(m_ui->actionSetTo1,        &QAction::triggered, this, &BewavedDolphin::on_actionSetTo1_triggered);
-    connect(m_ui->actionShowNumbers,   &QAction::triggered, this, &BewavedDolphin::on_actionShowNumbers_triggered);
-    connect(m_ui->actionShowWaveforms, &QAction::triggered, this, &BewavedDolphin::on_actionShowWaveforms_triggered);
-    connect(m_ui->actionZoomIn,        &QAction::triggered, this, &BewavedDolphin::on_actionZoomIn_triggered);
-    connect(m_ui->actionZoomOut,       &QAction::triggered, this, &BewavedDolphin::on_actionZoomOut_triggered);
+    connect(&m_view,                   &GraphicsView::scaleIn,  this, &BewavedDolphin::on_actionZoomIn_triggered);
+    connect(&m_view,                   &GraphicsView::scaleOut, this, &BewavedDolphin::on_actionZoomOut_triggered);
+    connect(m_ui->actionAbout,         &QAction::triggered,     this, &BewavedDolphin::on_actionAbout_triggered);
+    connect(m_ui->actionAboutQt,       &QAction::triggered,     this, &BewavedDolphin::on_actionAboutQt_triggered);
+    connect(m_ui->actionClear,         &QAction::triggered,     this, &BewavedDolphin::on_actionClear_triggered);
+    connect(m_ui->actionCombinational, &QAction::triggered,     this, &BewavedDolphin::on_actionCombinational_triggered);
+    connect(m_ui->actionCopy,          &QAction::triggered,     this, &BewavedDolphin::on_actionCopy_triggered);
+    connect(m_ui->actionCut,           &QAction::triggered,     this, &BewavedDolphin::on_actionCut_triggered);
+    connect(m_ui->actionExit,          &QAction::triggered,     this, &BewavedDolphin::on_actionExit_triggered);
+    connect(m_ui->actionExportToPdf,   &QAction::triggered,     this, &BewavedDolphin::on_actionExportToPdf_triggered);
+    connect(m_ui->actionExportToPng,   &QAction::triggered,     this, &BewavedDolphin::on_actionExportToPng_triggered);
+    connect(m_ui->actionFitScreen,     &QAction::triggered,     this, &BewavedDolphin::on_actionFitScreen_triggered);
+    connect(m_ui->actionInvert,        &QAction::triggered,     this, &BewavedDolphin::on_actionInvert_triggered);
+    connect(m_ui->actionLoad,          &QAction::triggered,     this, &BewavedDolphin::on_actionLoad_triggered);
+    connect(m_ui->actionPaste,         &QAction::triggered,     this, &BewavedDolphin::on_actionPaste_triggered);
+    connect(m_ui->actionResetZoom,     &QAction::triggered,     this, &BewavedDolphin::on_actionResetZoom_triggered);
+    connect(m_ui->actionSave,          &QAction::triggered,     this, &BewavedDolphin::on_actionSave_triggered);
+    connect(m_ui->actionSaveAs,        &QAction::triggered,     this, &BewavedDolphin::on_actionSaveAs_triggered);
+    connect(m_ui->actionSetClockWave,  &QAction::triggered,     this, &BewavedDolphin::on_actionSetClockWave_triggered);
+    connect(m_ui->actionSetLength,     &QAction::triggered,     this, &BewavedDolphin::on_actionSetLength_triggered);
+    connect(m_ui->actionSetTo0,        &QAction::triggered,     this, &BewavedDolphin::on_actionSetTo0_triggered);
+    connect(m_ui->actionSetTo1,        &QAction::triggered,     this, &BewavedDolphin::on_actionSetTo1_triggered);
+    connect(m_ui->actionShowNumbers,   &QAction::triggered,     this, &BewavedDolphin::on_actionShowNumbers_triggered);
+    connect(m_ui->actionShowWaveforms, &QAction::triggered,     this, &BewavedDolphin::on_actionShowWaveforms_triggered);
+    connect(m_ui->actionZoomIn,        &QAction::triggered,     this, &BewavedDolphin::on_actionZoomIn_triggered);
+    connect(m_ui->actionZoomOut,       &QAction::triggered,     this, &BewavedDolphin::on_actionZoomOut_triggered);
 }
 
 BewavedDolphin::~BewavedDolphin()
@@ -150,7 +151,7 @@ void BewavedDolphin::createWaveform(const QString &fileName)
         load(fileInfo.absoluteFilePath());
     }
 
-    qCDebug(zero) << tr("Resuming digital circuit main window after waveform simulation is finished.");
+    qCDebug(zero) << "Resuming digital circuit main window after waveform simulation is finished.";
     m_edited = false;
 }
 
@@ -202,13 +203,13 @@ void BewavedDolphin::loadFromTerminal()
 
 void BewavedDolphin::prepare(const QString &fileName)
 {
-    qCDebug(zero) << tr("Updating window name with current: ") << fileName;
+    qCDebug(zero) << "Updating window name with current: " << fileName;
     m_simulation = m_externalScene->simulation();
 
-    qCDebug(zero) << tr("Loading elements. All elements initially in elements vector. Then, inputs and outputs are extracted from it.");
+    qCDebug(zero) << "Loading elements. All elements initially in elements vector. Then, inputs and outputs are extracted from it.";
     loadElements();
 
-    qCDebug(zero) << tr("Loading initial data into the table.");
+    qCDebug(zero) << "Loading initial data into the table.";
     loadNewTable();
 }
 
@@ -254,15 +255,15 @@ void BewavedDolphin::loadElements()
 
 void BewavedDolphin::loadNewTable()
 {
-    qCDebug(zero) << tr("Getting initial value from inputs and writing them to oldvalues. Used to save current state of inputs and restore it after simulation. Not saving memory states though...");
-    qCDebug(zero) << tr("Also getting the name of the inputs. If no label is given, the element type is used as a name.");
+    qCDebug(zero) << "Getting initial value from inputs and writing them to oldvalues. Used to save current state of inputs and restore it after simulation. Not saving memory states though...";
+    qCDebug(zero) << "Also getting the name of the inputs. If no label is given, the element type is used as a name.";
     QStringList inputLabels;
     QStringList outputLabels;
     loadSignals(inputLabels, outputLabels);
 
     // ---------------------------------------
 
-    qCDebug(zero) << tr("Num iter = ") << m_length;
+    qCDebug(zero) << "Num iter = " << m_length;
 
     m_model = new SignalModel(inputLabels.size(), inputLabels.size() + outputLabels.size(), m_length, this);
     m_signalTableView->setModel(m_model);
@@ -277,20 +278,34 @@ void BewavedDolphin::loadNewTable()
 
     m_signalTableView->horizontalHeader()->setDefaultSectionSize(1);
 
-    qCDebug(zero) << tr("Inputs: ") << inputLabels.size() << tr(", outputs: ") << outputLabels.size();
+    qCDebug(zero) << "Inputs: " << inputLabels.size() << ", outputs: " << outputLabels.size();
 
     on_actionClear_triggered();
 
+    connect(m_signalTableView,                   &QAbstractItemView::doubleClicked,      this, &BewavedDolphin::on_tableView_cellDoubleClicked);
     connect(m_signalTableView->selectionModel(), &QItemSelectionModel::selectionChanged, this, &BewavedDolphin::on_tableView_selectionChanged);
 }
 
+void BewavedDolphin::on_tableView_cellDoubleClicked()
+{
+    const auto indexes = m_signalTableView->selectionModel()->selectedIndexes();
+
+    for (auto &index : indexes) {
+        int value = m_model->index(index.row(), index.column(), QModelIndex()).data().toInt();
+        value = (value + 1) % 2;
+        createElement(index.row(), index.column(), value);
+    }
+
+    run();
+}
+
 void BewavedDolphin::on_tableView_selectionChanged()
 {
     m_externalScene->clearSelection();
 
     const auto indexes = m_signalTableView->selectionModel()->selectedIndexes();
 
-    for (auto index : indexes) {
+    for (auto &index : indexes) {
         if (index.row() < m_inputs.size()) {
             m_inputs.at(index.row())->setSelected(true);
         }
@@ -304,7 +319,7 @@ void BewavedDolphin::loadSignals(QStringList &inputLabels, QStringList &outputLa
     QVector<Status> oldValues(m_inputPorts);
     int oldIndex = 0;
 
-    for (auto *input : qAsConst(m_inputs)) {
+    for (auto *input : std::as_const(m_inputs)) {
         QString label = input->label();
 
         if (label.isEmpty()) {
@@ -323,9 +338,9 @@ void BewavedDolphin::loadSignals(QStringList &inputLabels, QStringList &outputLa
         }
     }
 
-    qCDebug(zero) << tr("Getting the name of the outputs. If no label is given, element type is used as a name.");
+    qCDebug(zero) << "Getting the name of the outputs. If no label is given, element type is used as a name.";
 
-    for (auto *output : qAsConst(m_outputs)) {
+    for (auto *output : std::as_const(m_outputs)) {
         QString label = output->label();
 
         if (label.isEmpty()) {
@@ -352,45 +367,49 @@ void BewavedDolphin::run()
 
 void BewavedDolphin::run2()
 {
-    qCDebug(zero) << tr("Creating class to pause main window simulator while creating waveform.");
+    qCDebug(zero) << "Creating class to pause main window simulator while creating waveform.";
     SimulationBlocker simulationBlocker(m_simulation);
 
     for (int column = 0; column < m_model->columnCount(); ++column) {
-        qCDebug(four) << tr("Itr: ") << column << tr(", inputs: ") << m_inputs.size();
+        qCDebug(four) << "Itr: " << column << ", inputs: " << m_inputs.size();
         int row = 0;
 
-        for (auto *input : qAsConst(m_inputs)) {
+        for (auto *input : std::as_const(m_inputs)) {
+            const bool isRotary = dynamic_cast<InputRotary *>(input);
             for (int port = 0; port < input->outputSize(); ++port) {
-                const bool value = static_cast<bool>(m_model->index(row, column).data().toInt());
-                input->setOn(value, port);
-                ++row;
+                const bool value = m_model->index(row++, column).data().toBool();
+
+                if (isRotary && value) {
+                    input->setOn(1, port);
+                } else if (!isRotary) {
+                    input->setOn(value, port);
+                }
             }
         }
 
-        qCDebug(four) << tr("Updating the values of the circuit logic based on current input values.");
-        m_simulation->update();
+        qCDebug(four) << "Updating the values of the circuit logic based on current input values.";
         m_simulation->update();
         m_simulation->update();
 
-        qCDebug(four) << tr("Setting the computed output values to the waveform results.");
+        qCDebug(four) << "Setting the computed output values to the waveform results.";
         row = m_inputPorts;
 
-        for (auto *output : qAsConst(m_outputs)) {
+        for (auto *output : std::as_const(m_outputs)) {
             for (int port = 0; port < output->inputSize(); ++port) {
                 const int value = static_cast<int>(output->inputPort(port)->status());
                 createElement(row, column, value, false);
-                row++;
+                ++row;
             }
         }
     }
 
-    qCDebug(three) << tr("Setting inputs back to old values.");
+    qCDebug(three) << "Setting inputs back to old values.";
     restoreInputs();
 }
 
 void BewavedDolphin::restoreInputs()
 {
-    qCDebug(zero) << tr("Restoring old values to inputs, prior to simulation.");
+    qCDebug(zero) << "Restoring old values to inputs, prior to simulation.";
 
     for (int index = 0; index < m_inputs.size(); ++index) {
         for (int port = 0; port < m_inputs.value(index)->outputSize(); ++port) {
@@ -422,7 +441,8 @@ void BewavedDolphin::resizeScene()
         throw Pandaception(tr("Waveform would be too big! Resetting zoom."));
     }
 
-    m_signalTableView->resize(newWidth, newHeight);
+    m_signalTableView->resize(newWidth / (m_scale * 0.8),
+                              newHeight / (m_scale * 0.8));
     m_scene->setSceneRect(m_scene->itemsBoundingRect());
 }
 
@@ -445,11 +465,11 @@ bool BewavedDolphin::checkSave()
     auto reply =
             QMessageBox::question(
                 this,
-                tr("WiRedPanda - beWavedDolphin"),
+                tr("wiRedPanda - beWavedDolphin"),
                 tr("Save simulation before closing?"),
                 QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
 
-    switch(reply){
+    switch (reply) {
     case QMessageBox::Save:    on_actionSave_triggered(); return (!m_edited);
     case QMessageBox::Discard: return true;
     case QMessageBox::Cancel:  return false;
@@ -467,10 +487,10 @@ void BewavedDolphin::createZeroElement(const int row, const int col, const bool
 {
     const auto index = m_model->index(row, col);
 
-    qCDebug(three) << tr("Getting current value to check if need to refresh next cell");
+    qCDebug(three) << "Getting current value to check if need to refresh next cell";
     const int currentValue = index.data().toInt();
 
-    qCDebug(three) << tr("Changing current item.");
+    qCDebug(three) << "Changing current item.";
     m_model->setData(index, 0, Qt::DisplayRole);
 
     if (m_type == PlotType::Number) {
@@ -501,7 +521,7 @@ void BewavedDolphin::createZeroElement(const int row, const int col, const bool
         const auto nextIndex = m_model->index(row, col + 1);
 
         if (nextIndex.isValid() && (currentValue == 1)) {
-            qCDebug(three) << tr("Changing next item.");
+            qCDebug(three) << "Changing next item.";
             createElement(row, col + 1, nextIndex.data().toInt(), isInput, false);
         }
     }
@@ -511,10 +531,10 @@ void BewavedDolphin::createOneElement(const int row, const int col, const bool i
 {
     const auto index = m_model->index(row, col);
 
-    qCDebug(three) << tr("Getting current value to check if need to refresh next cell");
+    qCDebug(three) << "Getting current value to check if need to refresh next cell";
     const int currentValue = index.data().toInt();
 
-    qCDebug(three) << tr("Changing current item.");
+    qCDebug(three) << "Changing current item.";
     m_model->setData(index, 1, Qt::DisplayRole);
 
     if (m_type == PlotType::Number) {
@@ -545,7 +565,7 @@ void BewavedDolphin::createOneElement(const int row, const int col, const bool i
         const auto nextIndex = m_model->index(row, col + 1);
 
         if (nextIndex.isValid() && (currentValue == 0)) {
-            qCDebug(three) << tr("Changing next item.");
+            qCDebug(three) << "Changing next item.";
             createElement(row, col + 1, nextIndex.data().toInt(), isInput, false);
         }
     }
@@ -554,7 +574,7 @@ void BewavedDolphin::createOneElement(const int row, const int col, const bool i
 void BewavedDolphin::show()
 {
     QMainWindow::show();
-    qCDebug(zero) << tr("Getting table dimensions.");
+    qCDebug(zero) << "Getting table dimensions.";
     resizeScene();
 }
 
@@ -602,41 +622,41 @@ void BewavedDolphin::saveToTxt(QTextStream &stream)
 
 void BewavedDolphin::on_actionSetTo0_triggered()
 {
-    qCDebug(zero) << tr("Pressed 0.");
+    qCDebug(zero) << "Pressed 0.";
     const auto itemList = m_signalTableView->selectionModel()->selectedIndexes();
 
     for (const auto &item : itemList) {
         const int row = item.row();
         const int col = item.column();
-        qCDebug(zero) << tr("Editing value.");
+        qCDebug(zero) << "Editing value.";
         createZeroElement(row, col);
     }
 
     m_edited = true;
-    qCDebug(zero) << tr("Running simulation.");
+    qCDebug(zero) << "Running simulation.";
     run();
 }
 
 void BewavedDolphin::on_actionSetTo1_triggered()
 {
-    qCDebug(zero) << tr("Pressed 0.");
+    qCDebug(zero) << "Pressed 0.";
     const auto itemList = m_signalTableView->selectionModel()->selectedIndexes();
 
     for (const auto &item : itemList) {
         const int row = item.row();
         const int col = item.column();
-        qCDebug(zero) << tr("Editing value.");
+        qCDebug(zero) << "Editing value.";
         createOneElement(row, col);
     }
 
     m_edited = true;
-    qCDebug(zero) << tr("Running simulation.");
+    qCDebug(zero) << "Running simulation.";
     run();
 }
 
 void BewavedDolphin::on_actionInvert_triggered()
 {
-    qCDebug(zero) << tr("Pressed Not.");
+    qCDebug(zero) << "Pressed Not.";
     const auto itemList = m_signalTableView->selectionModel()->selectedIndexes();
 
     for (const auto &item : itemList) {
@@ -644,12 +664,12 @@ void BewavedDolphin::on_actionInvert_triggered()
         const int col = item.column();
         int value = m_model->index(row, col, QModelIndex()).data().toInt();
         value = (value + 1) % 2;
-        qCDebug(zero) << tr("Editing value.");
+        qCDebug(zero) << "Editing value.";
         createElement(row, col, value);
     }
 
     m_edited = true;
-    qCDebug(zero) << tr("Running simulation.");
+    qCDebug(zero) << "Running simulation.";
     run();
 }
 
@@ -681,7 +701,7 @@ int BewavedDolphin::sectionFirstRow(const QItemSelection &ranges)
 
 void BewavedDolphin::on_actionSetClockWave_triggered()
 {
-    qCDebug(zero) << tr("Getting first column.");
+    qCDebug(zero) << "Getting first column.";
     const auto ranges = m_signalTableView->selectionModel()->selection();
 
     if (ranges.isEmpty()) {
@@ -690,7 +710,7 @@ void BewavedDolphin::on_actionSetClockWave_triggered()
 
     const int firstCol = sectionFirstColumn(ranges);
 
-    qCDebug(zero) << tr("Setting the signal according to its column and clock period.");
+    qCDebug(zero) << "Setting the signal according to its column and clock period.";
     ClockDialog dialog(m_clockPeriod, this);
     const int clockPeriod = dialog.frequency();
 
@@ -707,24 +727,24 @@ void BewavedDolphin::on_actionSetClockWave_triggered()
         const int row = item.row();
         const int col = item.column();
         const int value = ((col - firstCol) % clockPeriod < halfClockPeriod ? 0 : 1);
-        qCDebug(zero) << tr("Editing value.");
+        qCDebug(zero) << "Editing value.";
         createElement(row, col, value);
     }
 
     m_edited = true;
-    qCDebug(zero) << tr("Running simulation.");
+    qCDebug(zero) << "Running simulation.";
     run();
 }
 
 void BewavedDolphin::on_actionCombinational_triggered()
 {
-    const int truthTableSize = std::pow(2, m_inputPorts);
+    const int truthTableSize = std::min(2048., std::pow(2, m_inputPorts));
 
     if (m_length < truthTableSize) {
         setLength(truthTableSize, false);
     }
 
-    qCDebug(zero) << tr("Setting the signal according to its columns and clock period.");
+    qCDebug(zero) << "Setting the signal according to its columns and clock period.";
     int halfClockPeriod = 1;
     int clockPeriod = 2;
 
@@ -739,13 +759,13 @@ void BewavedDolphin::on_actionCombinational_triggered()
     }
 
     m_edited = true;
-    qCDebug(zero) << tr("Running simulation.");
+    qCDebug(zero) << "Running simulation.";
     run();
 }
 
 void BewavedDolphin::on_actionSetLength_triggered()
 {
-    qCDebug(zero) << tr("Setting the simulation length.");
+    qCDebug(zero) << "Setting the simulation length.";
     const int currentLength = m_length > 0 ? m_length : m_model->columnCount();
     LengthDialog dialog(currentLength, this);
     const int simLength = dialog.length();
@@ -766,26 +786,26 @@ void BewavedDolphin::setLength(const int simLength, const bool runSimulation)
     m_length = simLength;
 
     if (simLength <= m_model->columnCount()) {
-        qCDebug(zero) << tr("Reducing or keeping the simulation length.");
+        qCDebug(zero) << "Reducing or keeping the simulation length.";
         m_model->setColumnCount(simLength);
         resizeScene();
         m_edited = true;
         return;
     }
 
-    qCDebug(zero) << tr("Increasing the simulation length.");
+    qCDebug(zero) << "Increasing the simulation length.";
     const int oldLength = m_model->columnCount();
     m_model->setColumnCount(simLength);
 
     for (int row = 0; row < m_inputPorts; ++row) {
-        for (int col = oldLength; col < m_model->columnCount(); ++col) {
+        for (int col = oldLength; col < simLength; ++col) {
             createZeroElement(row, col, true, false);
         }
     }
 
     resizeScene();
     m_edited = true;
-    qCDebug(zero) << tr("Running simulation.");
+    qCDebug(zero) << "Running simulation.";
 
     if (runSimulation) {
         run();
@@ -811,7 +831,7 @@ void BewavedDolphin::on_actionZoomIn_triggered()
 void BewavedDolphin::on_actionResetZoom_triggered()
 {
     m_view.resetZoom();
-    m_scale = 1.0;
+    m_scale = 1.25;
     resizeScene();
     zoomChanged();
 }
@@ -841,7 +861,7 @@ void BewavedDolphin::on_actionClear_triggered()
     }
 
     m_edited = true;
-    qCDebug(zero) << tr("Running simulation.");
+    qCDebug(zero) << "Running simulation.";
     run();
 }
 
@@ -856,18 +876,18 @@ void BewavedDolphin::on_actionCopy_triggered()
 
     QByteArray itemData;
     QDataStream stream(&itemData, QIODevice::WriteOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    Serialization::writeHeaderDolphin(stream);
     copy(ranges, stream);
 
     auto *mimeData = new QMimeData();
-    mimeData->setData("bdolphin/copydata", itemData);
+    mimeData->setData("application/x-bewaveddolphin-waveform", itemData);
 
     QApplication::clipboard()->setMimeData(mimeData);
 }
 
 void BewavedDolphin::copy(const QItemSelection &ranges, QDataStream &stream)
 {
-    qCDebug(zero) << tr("Serializing data into data stream.");
+    qCDebug(zero) << "Serializing data into data stream.";
     const int firstRow = sectionFirstRow(ranges);
     const int firstCol = sectionFirstColumn(ranges);
     const auto itemList = m_signalTableView->selectionModel()->selectedIndexes();
@@ -894,11 +914,11 @@ void BewavedDolphin::on_actionCut_triggered()
 
     QByteArray itemData;
     QDataStream stream(&itemData, QIODevice::WriteOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    Serialization::writeHeaderDolphin(stream);
     cut(ranges, stream);
 
     auto *mimeData = new QMimeData();
-    mimeData->setData("bdolphin/copydata", itemData);
+    mimeData->setData("application/x-bewaveddolphin-waveform", itemData);
 
     QApplication::clipboard()->setMimeData(mimeData);
 
@@ -920,15 +940,22 @@ void BewavedDolphin::on_actionPaste_triggered()
     }
 
     const auto *mimeData = QApplication::clipboard()->mimeData();
+    QByteArray itemData;
 
     if (mimeData->hasFormat("bdolphin/copydata")) {
-        QByteArray itemData = mimeData->data("bdolphin/copydata");
-        QDataStream stream(&itemData, QIODevice::ReadOnly);
-        stream.setVersion(QDataStream::Qt_5_12);
-        paste(ranges, stream);
+        itemData = mimeData->data("bdolphin/copydata");
     }
 
-    m_edited = true;
+    if (mimeData->hasFormat("application/x-bewaveddolphin-waveform")) {
+        itemData = mimeData->data("application/x-bewaveddolphin-waveform");
+    }
+
+    if (!itemData.isEmpty()) {
+        QDataStream stream(&itemData, QIODevice::ReadOnly);
+        Serialization::readHeaderDolphin(stream);
+        paste(ranges, stream);
+        m_edited = true;
+    }
 }
 
 void BewavedDolphin::paste(const QItemSelection &ranges, QDataStream &stream)
@@ -1001,7 +1028,7 @@ void BewavedDolphin::on_actionSaveAs_triggered()
 
     save(fileName);
     m_currentFile = QFileInfo(fileName);
-    associateToWiredPanda(fileName);
+    associateToWiRedPanda(fileName);
     setWindowTitle(tr("beWavedDolphin Simulator") + " [" + m_currentFile.fileName() + "]");
     m_ui->statusbar->showMessage(tr("Saved file successfully."), 4000);
     m_edited = false;
@@ -1016,12 +1043,12 @@ void BewavedDolphin::save(const QString &fileName)
     }
 
     if (fileName.endsWith(".dolphin")) {
-        qCDebug(zero) << tr("Saving dolphin file.");
+        qCDebug(zero) << "Saving dolphin file.";
         QDataStream stream(&file);
-        stream.setVersion(QDataStream::Qt_5_12);
+        Serialization::writeHeaderDolphin(stream);
         save(stream);
     } else {
-        qCDebug(zero) << tr("Saving CSV file.");
+        qCDebug(zero) << "Saving CSV file.";
         save(file);
     }
 
@@ -1032,8 +1059,7 @@ void BewavedDolphin::save(const QString &fileName)
 
 void BewavedDolphin::save(QDataStream &stream)
 {
-    stream << QString("beWavedDolphin 1.0");
-    qCDebug(zero) << tr("Serializing data into data stream.");
+    qCDebug(zero) << "Serializing data into data stream.";
     stream << static_cast<qint64>(m_inputPorts);
     stream << static_cast<qint64>(m_model->columnCount());
 
@@ -1063,14 +1089,14 @@ void BewavedDolphin::save(QSaveFile &file)
     }
 }
 
-void BewavedDolphin::associateToWiredPanda(const QString &fileName)
+void BewavedDolphin::associateToWiRedPanda(const QString &fileName)
 {
     if ((m_mainWindow->dolphinFileName() != fileName) && GlobalProperties::verbose) {
         const auto reply =
             QMessageBox::question(
                 this,
-                tr("WiRedPanda - beWavedDolphin"),
-                tr("Do you want to link this beWavedDolphin file to your current WiRedPanda file and save it?"),
+                tr("wiRedPanda - beWavedDolphin"),
+                tr("Do you want to link this beWavedDolphin file to your current wiRedPanda file and save it?"),
                 QMessageBox::Yes | QMessageBox::No);
 
         if (reply == QMessageBox::Yes) {
@@ -1126,46 +1152,40 @@ void BewavedDolphin::load(const QString &fileName)
         throw Pandaception(tr("File \"%1\" does not exist!").arg(fileName));
     }
 
-    qCDebug(zero) << tr("File exists.");
+    qCDebug(zero) << "File exists.";
 
     if (!file.open(QIODevice::ReadOnly)) {
-        qCDebug(zero) << tr("Could not open file in ReadOnly mode: ") << file.errorString();
+        qCDebug(zero) << "Could not open file in ReadOnly mode: " << file.errorString();
         throw Pandaception(tr("Could not open file in ReadOnly mode: ") + file.errorString() + ".");
     }
 
     if (fileName.endsWith(".dolphin")) {
-        qCDebug(zero) << tr("Dolphin file opened.");
+        qCDebug(zero) << "Dolphin file opened.";
         QDataStream stream(&file);
-        stream.setVersion(QDataStream::Qt_5_12);
-        qCDebug(zero) << tr("Loading in editor.");
+        Serialization::readHeaderDolphin(stream);
+        qCDebug(zero) << "Loading in editor.";
         load(stream);
-        qCDebug(zero) << tr("Current file set.");
+        qCDebug(zero) << "Current file set.";
         m_currentFile = QFileInfo(fileName);
     } else if (fileName.endsWith(".csv")) {
-        qCDebug(zero) << tr("CSV file opened.");
-        qCDebug(zero) << tr("Loading in editor.");
+        qCDebug(zero) << "CSV file opened.";
+        qCDebug(zero) << "Loading in editor.";
         load(file);
-        qCDebug(zero) << tr("Current file set.");
+        qCDebug(zero) << "Current file set.";
         m_currentFile = QFileInfo(fileName);
     } else {
-        qCDebug(zero) << tr("Format not supported. Could not open file: ") << fileName;
+        qCDebug(zero) << "Format not supported. Could not open file: " << fileName;
         throw Pandaception(tr("Format not supported. Could not open file: ") + fileName);
     }
 
-    qCDebug(zero) << tr("Closing file.");
+    qCDebug(zero) << "Closing file.";
     file.close();
-    associateToWiredPanda(fileName);
+    associateToWiRedPanda(fileName);
     setWindowTitle(tr("beWavedDolphin Simulator") + " [" + m_currentFile.fileName() + "]");
 }
 
 void BewavedDolphin::load(QDataStream &stream)
 {
-    QString str; stream >> str;
-
-    if (!str.startsWith("beWavedDolphin")) {
-        throw Pandaception(tr("Invalid file format. Starts with: ") + str);
-    }
-
     qint64 rows; stream >> rows;
     qint64 cols; stream >> cols;
 
@@ -1178,7 +1198,7 @@ void BewavedDolphin::load(QDataStream &stream)
     }
 
     setLength(static_cast<int>(cols), false);
-    qCDebug(zero) << tr("Update table.");
+    qCDebug(zero) << "Update table.";
 
     for (int col = 0; col < cols; ++col) {
         for (int row = 0; row < rows; ++row) {
@@ -1207,7 +1227,7 @@ void BewavedDolphin::load(QFile &file)
 
     setLength(cols, false);
 
-    qCDebug(zero) << tr("Update table.");
+    qCDebug(zero) << "Update table.";
 
     for (int row = 0; row < rows; ++row) {
         for (int col = 0; col < cols; ++col) {
@@ -1235,7 +1255,7 @@ void BewavedDolphin::on_actionShowNumbers_triggered()
         }
     }
 
-    qCDebug(zero) << tr("Running simulation.");
+    qCDebug(zero) << "Running simulation.";
     run();
 }
 
@@ -1249,7 +1269,7 @@ void BewavedDolphin::on_actionShowWaveforms_triggered()
         }
     }
 
-    qCDebug(zero) << tr("Running simulation.");
+    qCDebug(zero) << "Running simulation.";
     run();
 }
 
@@ -1309,14 +1329,14 @@ void BewavedDolphin::on_actionAbout_triggered()
 {
     QMessageBox::about(this,
         "beWavedDolphin",
-        tr("<p>beWavedDolphin is a waveform simulator for the WiRedPanda software developed by the Federal University of São Paulo."
+        tr("<p>beWavedDolphin is a waveform simulator for the wiRedPanda software developed by the Federal University of São Paulo."
            " This project was created in order to help students learn about logic circuits.</p>"
            "<p>Software version: %1</p>"
            "<p><strong>Creators:</strong></p>"
            "<ul>"
            "<li> Prof. Fábio Cappabianco, Ph.D. </li>"
            "</ul>"
-           "<p> beWavedDolphin is currently maintained by Prof. Fábio Cappabianco, Ph.D. and Vinícius R. Miguel.</p>"
+           "<p> beWavedDolphin is currently maintained by Prof. Fábio Cappabianco, Ph.D. and his students</p>"
            "<p> Please file a report at our GitHub page if bugs are found or if you wish for a new functionality to be implemented.</p>"
            "<p><a href=\"http://gibis-unifesp.github.io/wiRedPanda/\">Visit our website!</a></p>")
             .arg(QApplication::applicationVersion()));
