@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
@@ -71,7 +71,7 @@ private:
     bool checkSave();
     int sectionFirstColumn(const QItemSelection &ranges);
     int sectionFirstRow(const QItemSelection &ranges);
-    void associateToWiredPanda(const QString &fileName);
+    void associateToWiRedPanda(const QString &fileName);
     void copy(const QItemSelection &ranges, QDataStream &stream);
     void createElement(const int row, const int col, const int value, const bool isInput = true, const bool changeNext = true);
     void createOneElement(const int row, const int col, const bool isInput = true, const bool changeNext = true);
@@ -109,6 +109,7 @@ private:
     void on_actionShowWaveforms_triggered();
     void on_actionZoomIn_triggered();
     void on_actionZoomOut_triggered();
+    void on_tableView_cellDoubleClicked();
     void on_tableView_selectionChanged();
     void paste(const QItemSelection &ranges, QDataStream &stream);
     void prepare(const QString &fileName = {});
@@ -146,7 +147,7 @@ private:
     bool m_edited = false;
     const bool m_askConnection;
     const double m_scaleFactor = 0.8;
-    double m_scale = 1.0;
+    double m_scale = 1.25;
     int m_clockPeriod = 0;
     int m_inputPorts = 0;
     int m_length = 32;
