@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "commands.h"
@@ -13,6 +13,7 @@
 #include "serialization.h"
 #include "simulation.h"
 #include "simulationblocker.h"
+#include "truth_table.h"
 
 #include <QIODevice>
 #include <cmath>
@@ -143,7 +144,8 @@ void saveItems(QByteArray &itemData, const QList<QGraphicsItem *> &items, const
 {
     itemData.clear();
     QDataStream stream(&itemData, QIODevice::WriteOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    Serialization::writeHeaderPanda(stream);
+
     const auto others = findElements(otherIds);
 
     for (auto *elm : others) {
@@ -173,9 +175,9 @@ const QList<QGraphicsItem *> loadItems(Scene *scene, QByteArray &itemData, const
     }
 
     QDataStream stream(&itemData, QIODevice::ReadOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    QVersionNumber version = Serialization::readHeaderPanda(stream);
+
     QMap<quint64, QNEPort *> portMap;
-    const QVersionNumber version = GlobalProperties::version;
 
     for (auto *elm : findElements(otherIds)) {
         elm->load(stream, portMap, version);
@@ -306,7 +308,7 @@ void RotateCommand::redo()
     for (auto *elm : elements) {
         cx += elm->pos().x();
         cy += elm->pos().y();
-        sz++;
+        ++sz;
     }
 
     if (sz != 0) {
@@ -374,7 +376,7 @@ UpdateCommand::UpdateCommand(const QList<GraphicElement *> &elements, const QByt
 {
     m_ids.reserve(elements.size());
     QDataStream stream(&m_newData, QIODevice::WriteOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    Serialization::writeHeaderPanda(stream);
 
     for (auto *elm : elements) {
         elm->save(stream);
@@ -407,9 +409,9 @@ void UpdateCommand::loadData(QByteArray &itemData)
     }
 
     QDataStream stream(&itemData, QIODevice::ReadOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    QVersionNumber version = Serialization::readHeaderPanda(stream);
+
     QMap<quint64, QNEPort *> portMap;
-    const QVersionNumber version = GlobalProperties::version;
 
     for (auto *elm : elements) {
         elm->load(stream, portMap, version);
@@ -699,8 +701,9 @@ void ChangeInputSizeCommand::redo()
     QList<GraphicElement *> serializationOrder;
     serializationOrder.reserve(m_elements.size());
     m_oldData.clear();
+
     QDataStream stream(&m_oldData, QIODevice::WriteOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    Serialization::writeHeaderPanda(stream);
 
     for (auto *elm : m_elements) {
         elm->save(stream);
@@ -746,9 +749,9 @@ void ChangeInputSizeCommand::undo()
     const auto serializationOrder = findElements(m_order);
 
     QDataStream stream(&m_oldData, QIODevice::ReadOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    QVersionNumber version = Serialization::readHeaderPanda(stream);
+
     QMap<quint64, QNEPort *> portMap;
-    const QVersionNumber version = GlobalProperties::version;
 
     for (auto *elm : serializationOrder) {
         elm->load(stream, portMap, version);
@@ -787,10 +790,11 @@ void ChangeOutputSizeCommand::redo()
     const auto m_elements = findElements(m_ids);
 
     QList<GraphicElement *> serializationOrder;
-    m_oldData.clear();
-    QDataStream stream(&m_oldData, QIODevice::WriteOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
     serializationOrder.reserve(m_elements.size());
+    m_oldData.clear();
+
+    QDataStream stream(&m_oldData, QIODevice::WriteOnly);
+    Serialization::writeHeaderPanda(stream);
 
     for (auto *elm : m_elements) {
         elm->save(stream);
@@ -837,9 +841,9 @@ void ChangeOutputSizeCommand::undo()
     const auto serializationOrder = findElements(m_order);
 
     QDataStream stream(&m_oldData, QIODevice::ReadOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    QVersionNumber version = Serialization::readHeaderPanda(stream);
+
     QMap<quint64, QNEPort *> portMap;
-    const QVersionNumber version = GlobalProperties::version;
 
     for (auto *elm : serializationOrder) {
         elm->load(stream, portMap, version);
@@ -857,3 +861,41 @@ void ChangeOutputSizeCommand::undo()
 
     m_scene->setCircuitUpdateRequired();
 }
+
+ToggleTruthTableOutputCommand::ToggleTruthTableOutputCommand(GraphicElement *element, int pos, Scene *scene, ElementEditor *elementeditor, QUndoCommand *parent)
+    : QUndoCommand(parent)
+    , m_pos(pos)
+{
+    m_id = element->id();
+    m_scene = scene;
+    m_elementeditor = elementeditor;
+    setText(tr("Toggle TruthTable Output at position: %1").arg(m_pos));
+}
+
+void ToggleTruthTableOutputCommand::redo()
+{
+    qCDebug(zero) << text();
+
+    auto *truthtable = dynamic_cast<TruthTable *>(findElm(m_id));
+
+    if (!truthtable) throw Pandaception("Could not find truthtable element!");
+
+    truthtable->key().toggleBit(m_pos);
+
+    m_scene->setCircuitUpdateRequired();
+    m_elementeditor->truthTable();
+}
+
+void ToggleTruthTableOutputCommand::undo()
+{
+    qCDebug(zero) << text();
+
+    auto *truthtable = dynamic_cast<TruthTable *>(findElm(m_id));
+
+    if (!truthtable) throw Pandaception("Could not find truthtable element!");
+
+    truthtable->key().toggleBit(m_pos);
+
+    m_scene->setCircuitUpdateRequired();
+    m_elementeditor->truthTable();
+}
