@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "inputrotary.h"
@@ -9,11 +9,6 @@
 #include <QGraphicsSceneMouseEvent>
 #include <QPainter>
 
-namespace
-{
-    int id = qRegisterMetaType<InputRotary>();
-}
-
 InputRotary::InputRotary(QGraphicsItem *parent)
     : GraphicElementInput(ElementType::InputRotary, ElementGroup::Input, ":/input/rotary_icon.svg", tr("ROTARY SWITCH"), tr("Rotary Switch"), 0, 0, 2, 16, parent)
 {
