@@ -1,18 +1,22 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "elementeditor.h"
 #include "ui_elementeditor.h"
 
+#include "audiobox.h"
 #include "commands.h"
 #include "common.h"
 #include "elementfactory.h"
 #include "inputrotary.h"
 #include "scene.h"
+#include "serialization.h"
 #include "thememanager.h"
+#include "truth_table.h"
 
 #include <QDebug>
 #include <QFileDialog>
+#include <QImageReader>
 #include <QKeyEvent>
 #include <QMenu>
 #include <cmath>
@@ -34,22 +38,36 @@ ElementEditor::ElementEditor(QWidget *parent)
     m_ui->comboBoxInputSize->installEventFilter(this);
     m_ui->comboBoxOutputSize->installEventFilter(this);
     m_ui->comboBoxValue->installEventFilter(this);
+    m_ui->doubleSpinBoxDelay->installEventFilter(this);
     m_ui->doubleSpinBoxFrequency->installEventFilter(this);
     m_ui->lineEditElementLabel->installEventFilter(this);
     m_ui->lineEditTrigger->installEventFilter(this);
     m_ui->spinBoxPriority->installEventFilter(this);
 
+    m_tableBox = new QDialog(this);
+    m_tableBox->setWindowFlags(Qt::Window);
+    m_tableBox->setWindowTitle(tr("Truth Table"));
+    m_tableBox->setModal(true);
+    auto *tableLayout = new QGridLayout(m_tableBox);
+    m_table = new QTableWidget(this);
+    tableLayout->addWidget(m_table);
+    m_tableBox->setLayout(tableLayout);
+
+    connect(m_table,                      &QTableWidget::cellDoubleClicked,                 this, &ElementEditor::setTruthTableProposition);
     connect(m_ui->checkBoxLocked,         &QCheckBox::clicked,                              this, &ElementEditor::inputLocked);
     connect(m_ui->comboBoxAudio,          qOverload<int>(&QComboBox::currentIndexChanged),  this, &ElementEditor::apply);
     connect(m_ui->comboBoxColor,          qOverload<int>(&QComboBox::currentIndexChanged),  this, &ElementEditor::apply);
     connect(m_ui->comboBoxInputSize,      qOverload<int>(&QComboBox::currentIndexChanged),  this, &ElementEditor::inputIndexChanged);
     connect(m_ui->comboBoxOutputSize,     qOverload<int>(&QComboBox::currentIndexChanged),  this, &ElementEditor::outputIndexChanged);
     connect(m_ui->comboBoxValue,          &QComboBox::currentTextChanged,                   this, &ElementEditor::outputValueChanged);
+    connect(m_ui->doubleSpinBoxDelay,     qOverload<double>(&QDoubleSpinBox::valueChanged), this, &ElementEditor::apply);
     connect(m_ui->doubleSpinBoxFrequency, qOverload<double>(&QDoubleSpinBox::valueChanged), this, &ElementEditor::apply);
     connect(m_ui->lineEditElementLabel,   &QLineEdit::textChanged,                          this, &ElementEditor::apply);
     connect(m_ui->lineEditTrigger,        &QLineEdit::textChanged,                          this, &ElementEditor::triggerChanged);
+    connect(m_ui->pushButtonAudioBox,     &QPushButton::clicked,                            this, &ElementEditor::audioBox);
     connect(m_ui->pushButtonChangeSkin,   &QPushButton::clicked,                            this, &ElementEditor::updateElementSkin);
     connect(m_ui->pushButtonDefaultSkin,  &QPushButton::clicked,                            this, &ElementEditor::defaultSkin);
+    connect(m_ui->pushButtonTruthTable,   &QPushButton::clicked,                            this, &ElementEditor::truthTable);
     connect(m_ui->spinBoxPriority,        qOverload<int>(&QSpinBox::valueChanged),          this, &ElementEditor::priorityChanged);
 }
 
@@ -299,7 +317,7 @@ void ElementEditor::updateElementSkin()
     QFileDialog fileDialog;
     fileDialog.setObjectName(tr("Open File"));
     fileDialog.setFileMode(QFileDialog::ExistingFile);
-    fileDialog.setNameFilter(tr("Images (*.png *.gif *.jpg *.jpeg)"));
+    fileDialog.setNameFilter(tr("Images") + " (*." + QImageReader::supportedImageFormats().join(" *.") + ")");
 
     if (fileDialog.exec() == QDialog::Rejected) {
         return;
@@ -312,7 +330,7 @@ void ElementEditor::updateElementSkin()
         return;
     }
 
-    qCDebug(zero) << tr("File name: ") << fileName;
+    qCDebug(zero) << "File name: " << fileName;
     m_isUpdatingSkin = true;
     m_skinName = fileName;
     m_isDefaultSkin = false;
@@ -360,8 +378,8 @@ void ElementEditor::retranslateUi()
 void ElementEditor::setCurrentElements(const QList<GraphicElement *> &elements)
 {
     m_elements = elements;
-    m_hasLabel = m_hasColors = m_hasAudio = m_hasFrequency = m_canChangeInputSize = m_canChangeOutputSize = m_hasTrigger = false;
-    m_hasRotation = m_hasSameLabel = m_hasSameColors = m_hasSameFrequency = m_hasSameAudio = m_hasOnlyInputs = false;
+    m_hasAudioBox = m_hasTruthTable = m_hasLabel = m_hasColors = m_hasAudio = m_hasFrequency = m_hasDelay = m_canChangeInputSize = m_canChangeOutputSize = m_hasTrigger = false;
+    m_hasRotation = m_hasSameLabel = m_hasSameColors = m_hasSameFrequency = m_hasSameDelay = m_hasSameAudio = m_hasOnlyInputs = false;
     m_hasSameInputSize = m_hasSameOutputSize = m_hasSameOutputValue = m_hasSameTrigger = m_canMorph = m_hasSameType = false;
     m_canChangeSkin = m_hasSamePriority = false;
     m_hasElements = false;
@@ -373,11 +391,13 @@ void ElementEditor::setCurrentElements(const QList<GraphicElement *> &elements)
     }
 
     bool sameCheckState = true;
-    m_hasLabel = m_hasColors = m_hasAudio = m_hasFrequency = m_canChangeInputSize = m_canChangeOutputSize = m_hasTrigger = true;
+    m_hasTruthTable = m_hasLabel = m_hasColors = m_hasAudio = m_hasFrequency = m_hasDelay = m_canChangeInputSize = m_canChangeOutputSize = m_hasTrigger = true;
+    m_hasRotation = m_hasSameLabel = m_hasSameColors = m_hasSameFrequency = m_hasSameDelay = m_hasSameAudio = m_hasOnlyInputs = true;
     m_hasSameInputSize = m_hasSameOutputSize = m_hasSameOutputValue = m_hasSameTrigger = m_canMorph = m_hasSameType = true;
-    m_hasRotation = m_hasSameLabel = m_hasSameColors = m_hasSameFrequency = m_hasSameAudio = m_hasOnlyInputs = true;
     m_canChangeSkin = m_hasSamePriority = true;
     m_hasElements = true;
+    m_hasAudioBox = true;
+
     show();
     setEnabled(false);
     int minimumInputs = 0;
@@ -389,15 +409,18 @@ void ElementEditor::setCurrentElements(const QList<GraphicElement *> &elements)
     auto *firstInput = qobject_cast<GraphicElementInput *>(firstElement);
     auto elementType = firstElement->elementType();
 
-    for (auto *elm : qAsConst(m_elements)) {
+    for (auto *elm : std::as_const(m_elements)) {
         const auto group = elm->elementGroup();
         const auto firstGroup = firstElement->elementGroup();
 
+        m_hasTruthTable &= elm->hasTruthTable();
         m_hasLabel &= elm->hasLabel();
         m_canChangeSkin &= elm->canChangeSkin();
         m_hasColors &= elm->hasColors();
         m_hasAudio &= elm->hasAudio();
+        m_hasAudioBox &= elm->hasAudioBox();
         m_hasFrequency &= elm->hasFrequency();
+        m_hasDelay &= elm->hasDelay();
         minimumInputs = std::max(minimumInputs, elm->minInputSize());
         maximumInputs = std::min(maximumInputs, elm->maxInputSize());
         minimumOutputs = std::max(minimumOutputs, elm->minOutputSize());
@@ -408,6 +431,7 @@ void ElementEditor::setCurrentElements(const QList<GraphicElement *> &elements)
         m_hasSameLabel &= (elm->label() == firstElement->label());
         m_hasSameColors &= (elm->color() == firstElement->color());
         m_hasSameFrequency &= qFuzzyCompare(elm->frequency(), firstElement->frequency());
+        m_hasSameDelay &= qFuzzyCompare(elm->delay(), firstElement->delay());
         m_hasSameInputSize &= (elm->inputSize() == firstElement->inputSize());
         m_hasSameOutputSize &= (elm->outputSize() == firstElement->outputSize());
         maxCurrentOutputSize = std::min(maxCurrentOutputSize, elm->outputSize());
@@ -434,7 +458,7 @@ void ElementEditor::setCurrentElements(const QList<GraphicElement *> &elements)
     }
 
     m_canChangeInputSize = (minimumInputs < maximumInputs);
-    m_canChangeOutputSize = (minimumOutputs < maximumOutputs);
+    m_canChangeOutputSize = (minimumOutputs < maximumOutputs && !m_hasTruthTable);
     /* Element type */
     m_ui->groupBox->setTitle(ElementFactory::typeToTitleText(elementType));
     /* Labels */
@@ -494,6 +518,10 @@ void ElementEditor::setCurrentElements(const QList<GraphicElement *> &elements)
     m_ui->doubleSpinBoxFrequency->setEnabled(m_hasFrequency);
     m_ui->labelFrequency->setVisible(m_hasFrequency);
 
+    m_ui->doubleSpinBoxDelay->setVisible(m_hasDelay);
+    m_ui->doubleSpinBoxDelay->setEnabled(m_hasDelay);
+    m_ui->labelDelay->setVisible(m_hasDelay);
+
     if (m_hasFrequency) {
         if (m_hasSameFrequency) {
             m_ui->doubleSpinBoxFrequency->setMinimum(0.1);
@@ -506,6 +534,18 @@ void ElementEditor::setCurrentElements(const QList<GraphicElement *> &elements)
         }
     }
 
+    if (m_hasDelay) {
+        if (m_hasSameDelay) {
+            m_ui->doubleSpinBoxDelay->setMinimum(0.0);
+            m_ui->doubleSpinBoxDelay->setSpecialValueText({});
+            m_ui->doubleSpinBoxDelay->setValue(static_cast<double>(firstElement->delay()));
+        } else {
+            m_ui->doubleSpinBoxDelay->setMinimum(0.0);
+            m_ui->doubleSpinBoxDelay->setSpecialValueText(m_manyDelay);
+            m_ui->doubleSpinBoxDelay->setValue(0.0);
+        }
+    }
+
     /* Input size */
     m_ui->comboBoxInputSize->clear();
     m_ui->labelInputs->setVisible(m_canChangeInputSize);
@@ -551,7 +591,7 @@ void ElementEditor::setCurrentElements(const QList<GraphicElement *> &elements)
         m_ui->comboBoxOutputSize->addItem(m_manyOS);
     }
 
-    if (m_canChangeOutputSize) {
+    if (m_canChangeOutputSize && !m_hasTruthTable) {
         if (m_hasSameOutputSize) {
             m_ui->comboBoxOutputSize->removeItem(m_ui->comboBoxOutputSize->findText(m_manyOS));
             QString outputSize = QString::number(firstElement->outputSize());
@@ -569,7 +609,7 @@ void ElementEditor::setCurrentElements(const QList<GraphicElement *> &elements)
 
     if (m_hasOnlyInputs) {
         if (maxCurrentOutputSize == 1) {
-            maxCurrentOutputSize++;
+            ++maxCurrentOutputSize;
         }
 
         for (int val = 0; val < maxCurrentOutputSize; ++val) {
@@ -613,6 +653,24 @@ void ElementEditor::setCurrentElements(const QList<GraphicElement *> &elements)
         m_ui->lineEditTrigger->setText(m_hasSameTrigger ? firstElement->trigger().toString() : m_manyTriggers);
     }
 
+    /* AudioBox */
+    m_ui->pushButtonAudioBox->setVisible(m_hasAudioBox);
+    m_ui->pushButtonAudioBox->setEnabled(m_hasAudioBox);
+    m_ui->labelAudioBox->setVisible(m_hasAudioBox);
+    m_ui->lineCurrentAudioBox->setVisible(m_hasAudioBox);
+
+    if (m_hasAudioBox) {
+        if (elements.size() > 1) {
+            m_ui->lineCurrentAudioBox->setText(m_manyAudios);
+        }
+
+        m_ui->lineCurrentAudioBox->setText(elements[0]->audio());
+    }
+
+    /* TruthTable */
+    m_ui->pushButtonTruthTable->setVisible(m_hasTruthTable);
+    m_ui->pushButtonTruthTable->setEnabled(m_hasTruthTable);
+
     setEnabled(true);
     show();
 
@@ -628,7 +686,7 @@ void ElementEditor::selectionChanged()
 
 void ElementEditor::apply()
 {
-    qCDebug(three) << tr("Apply.");
+    qCDebug(three) << "Apply.";
 
     if (m_elements.isEmpty() || !isEnabled()) {
         return;
@@ -636,9 +694,9 @@ void ElementEditor::apply()
 
     QByteArray oldData;
     QDataStream stream(&oldData, QIODevice::WriteOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    Serialization::writeHeaderPanda(stream);
 
-    for (auto *elm : qAsConst(m_elements)) {
+    for (auto *elm : std::as_const(m_elements)) {
         elm->save(stream);
 
         elm->setPriority(m_ui->spinBoxPriority->value());
@@ -659,6 +717,10 @@ void ElementEditor::apply()
             elm->setFrequency(static_cast<float>(m_ui->doubleSpinBoxFrequency->value()));
         }
 
+        if (elm->hasDelay() && (m_ui->doubleSpinBoxDelay->text() != m_manyDelay)) {
+            elm->setDelay(static_cast<float>(m_ui->doubleSpinBoxDelay->value()));
+        }
+
         if (elm->hasTrigger() && (m_ui->lineEditTrigger->text() != m_manyTriggers)) {
             if (m_ui->lineEditTrigger->text().size() <= 1) {
                 elm->setTrigger(QKeySequence(m_ui->lineEditTrigger->text()));
@@ -687,7 +749,7 @@ void ElementEditor::inputIndexChanged(const int index)
         emit sendCommand(new ChangeInputSizeCommand(m_elements, m_ui->comboBoxInputSize->currentData().toInt(), m_scene));
     }
 
-    qCDebug(zero) << tr("Input size changed to ") << index;
+    qCDebug(zero) << "Input size changed to " << index;
     update();
 }
 
@@ -701,7 +763,7 @@ void ElementEditor::outputIndexChanged(const int index)
         emit sendCommand(new ChangeOutputSizeCommand(m_elements, m_ui->comboBoxOutputSize->currentData().toInt(), m_scene));
     }
 
-    qCDebug(zero) << tr("Output size changed to ") << index;
+    qCDebug(zero) << "Output size changed to " << index;
     update();
 }
 
@@ -713,7 +775,7 @@ void ElementEditor::outputValueChanged(const QString &value)
 
     const int newValue = value.toInt();
 
-    for (auto *elm : qAsConst(m_elements)) {
+    for (auto *elm : std::as_const(m_elements)) {
         if (elm->elementType() == ElementType::InputRotary) {
             if (auto *input = qobject_cast<InputRotary *>(elm)) {
                 input->setOn(true, newValue);
@@ -734,13 +796,13 @@ void ElementEditor::inputLocked(const bool value)
         return;
     }
 
-    for (auto *elm : qAsConst(m_elements)) {
+    for (auto *elm : std::as_const(m_elements)) {
         if (auto *input = qobject_cast<GraphicElementInput *>(elm)) {
             input->setLocked(value);
         }
     }
 
-    qCDebug(zero) << tr("Input locked.");
+    qCDebug(zero) << "Input locked.";
     apply();
 }
 
@@ -778,7 +840,7 @@ bool ElementEditor::eventFilter(QObject *obj, QEvent *event)
                 });
 
                 const int elmPos = elements.indexOf(elm);
-                qCDebug(zero) << tr("Pos = ") << elmPos << tr(" from ") << elements.size();
+                qCDebug(zero) << "Pos = " << elmPos << " from " << elements.size();
                 int step = 1;
 
                 if (moveBack) {
@@ -789,7 +851,7 @@ bool ElementEditor::eventFilter(QObject *obj, QEvent *event)
                 int pos = (elements.size() + elmPos + step) % elements.size();
 
                 for (; pos != elmPos; pos = ((elements.size() + pos + step) % elements.size())) {
-                    qCDebug(zero) << tr("Pos = ") << pos;
+                    qCDebug(zero) << "Pos = " << pos;
                     elm = elements.at(pos);
 
                     setCurrentElements({elm});
@@ -817,6 +879,103 @@ bool ElementEditor::eventFilter(QObject *obj, QEvent *event)
     return QWidget::eventFilter(obj, event);
 }
 
+void ElementEditor::truthTable()
+{
+    if (!m_hasTruthTable) return;
+
+    auto *truthtable = dynamic_cast<TruthTable *>(m_elements[0]);
+
+    // Assuming only one element selected for now...
+
+    int nInputs = truthtable->inputSize();
+    int nOutputs = truthtable->outputSize();
+
+    QStringList inputLabels;
+    m_table->setColumnCount(nInputs + nOutputs);
+    m_table->setRowCount(std::pow(2, nInputs));
+
+    for (int i = 0; i < nInputs; ++i) {
+        inputLabels.append(QChar::fromLatin1('A' + i));
+    }
+
+    for (int i = 0; i < truthtable->outputSize(); ++i) {
+        inputLabels.append("S" + QString::number(i));
+        m_table->setColumnWidth(nInputs + i, 14);
+    }
+
+    m_table->setHorizontalHeaderLabels(inputLabels);
+
+    for (int i = 0; i < std::pow(2, nInputs); ++i) {
+        for (int j = 0; j < nInputs; ++j) {
+            m_table->setColumnWidth(j, 14);
+            auto newItemValue = QString::number(i, 2);
+
+            if (newItemValue.size() < nInputs) {
+                newItemValue = newItemValue.rightJustified(nInputs, '0');
+            }
+
+            if (m_table->item(i, j) == nullptr) {
+                auto *newItem = new QTableWidgetItem(newItemValue.at(j), QTableWidgetItem::Type);
+                newItem->setTextAlignment(Qt::AlignCenter);
+                m_table->setItem(i, j, newItem);
+                m_table->item(i, j)->setFlags(Qt::ItemIsEnabled);
+            }
+
+            m_table->item(i, j)->setText(newItemValue.at(j));
+        }
+
+        auto bitArray = truthtable->key();
+
+        for (int z = 0; z < nOutputs; ++z) {
+            const int output = bitArray.at(256 * z + i);
+
+            if (m_table->item(i, nInputs + z) == nullptr) {
+                auto *newOutItem = new QTableWidgetItem(QString(QChar::fromLatin1('0' + output)));
+                newOutItem->setTextAlignment(Qt::AlignCenter);
+                m_table->setItem(i, nInputs + z, newOutItem);
+                m_table->item(i, nInputs + z)->setFlags(Qt::ItemIsEnabled);
+            }
+
+            m_table->item(i, nInputs + z)->setText(QString::number(output));
+        }
+    }
+
+    m_tableBox->show();
+}
+
+void ElementEditor::setTruthTableProposition(const int row, const int column)
+{
+    if (m_elements.size() > 1) return;
+
+    if (column < m_elements[0]->inputSize()) return;
+
+    auto cellItem = m_table->item(row, column);
+    cellItem->setText((cellItem->text() == "0") ? "1" : "0");
+
+    auto truthtable = m_elements[0];
+    const int nInputs = truthtable->inputSize();
+    const int positionToChange = 256 * (column - nInputs) + row;
+
+    emit sendCommand(new ToggleTruthTableOutputCommand(truthtable, positionToChange, m_scene, this));
+
+    ElementEditor::truthTable();
+
+    update();
+
+    m_scene->setCircuitUpdateRequired();
+}
+
+void ElementEditor::audioBox()
+{
+    auto *audiobox = dynamic_cast<AudioBox *>(m_elements[0]);
+
+    const QString filePath = QFileDialog::getOpenFileName(this, tr("Select any audio"),
+                                                    QString(), tr("Audio (*.mp3 *.mp4 *.wav *.ogg)"));
+
+    audiobox->setAudio(filePath);
+    m_ui->lineCurrentAudioBox->setText(QFileInfo(filePath).fileName());
+}
+
 void ElementEditor::defaultSkin()
 {
     m_isUpdatingSkin = true;
