@@ -1,10 +1,13 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
 #include "scene.h"
 
+#include <QDialog>
+#include <QTableWidget>
+#include <QVBoxLayout>
 #include <QWidget>
 
 namespace Ui
@@ -21,12 +24,14 @@ public:
     ~ElementEditor() override;
 
     bool eventFilter(QObject *obj, QEvent *event) override;
+    void audioBox();
     void changeTriggerAction();
     void contextMenu(QPoint screenPos, QGraphicsItem *itemAtMouse);
     void fillColorComboBox();
     void renameAction();
     void retranslateUi();
     void setScene(Scene *scene);
+    void truthTable();
     void update();
     void updateElementSkin();
     void updatePriorityAction();
@@ -44,16 +49,19 @@ private:
     void inputLocked(const bool value);
     void outputIndexChanged(const int index);
     void outputValueChanged(const QString &value);
+    void priorityChanged(const int value);
     void selectionChanged();
     void setCurrentElements(const QList<GraphicElement *> &elements);
+    void setTruthTableProposition(const int row, const int column);
     void triggerChanged(const QString &cmd);
-    void priorityChanged(const int value);
     void updateSkins();
 
     Ui::ElementEditor *m_ui;
+    QDialog *m_tableBox = nullptr;
     QList<GraphicElement *> m_elements;
     QString m_manyAudios = tr("<Many sounds>");
     QString m_manyColors = tr("<Many colors>");
+    QString m_manyDelay = tr("<Many values>");
     QString m_manyFreq = tr("<Many values>");
     QString m_manyIS = tr("<Many values>");
     QString m_manyLabels = tr("<Many labels>");
@@ -62,6 +70,7 @@ private:
     QString m_manyPriorities = tr("<Many priorities>");
     QString m_manyTriggers = tr("<Many triggers>");
     QString m_skinName;
+    QTableWidget *m_table = nullptr;
     Scene *m_scene = nullptr;
     bool m_canChangeInputSize = false;
     bool m_canChangeOutputSize = false;
@@ -69,7 +78,9 @@ private:
     bool m_canMorph = false;
     bool m_hasAnyProperty = false;
     bool m_hasAudio = false;
+    bool m_hasAudioBox = false;
     bool m_hasColors = false;
+    bool m_hasDelay = false;
     bool m_hasElements = false;
     bool m_hasFrequency = false;
     bool m_hasLabel = false;
@@ -77,6 +88,7 @@ private:
     bool m_hasRotation = false;
     bool m_hasSameAudio = false;
     bool m_hasSameColors = false;
+    bool m_hasSameDelay = false;
     bool m_hasSameFrequency = false;
     bool m_hasSameInputSize = false;
     bool m_hasSameLabel = false;
@@ -86,6 +98,7 @@ private:
     bool m_hasSameTrigger = false;
     bool m_hasSameType = false;
     bool m_hasTrigger = false;
+    bool m_hasTruthTable = false;
     bool m_isDefaultSkin = true;
     bool m_isUpdatingSkin = false;
 };
