@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "elementfactory.h"
@@ -21,9 +21,12 @@
 #include "logicor.h"
 #include "logicoutput.h"
 #include "logicsrflipflop.h"
+#include "logicsrlatch.h"
 #include "logictflipflop.h"
+#include "logictruthtable.h"
 #include "logicxnor.h"
 #include "logicxor.h"
+#include "truth_table.h"
 
 #include <QMetaEnum>
 
@@ -158,6 +161,7 @@ std::shared_ptr<LogicElement> ElementFactory::buildLogicElement(GraphicElement *
     case ElementType::InputRotary:
     case ElementType::InputSwitch: return std::make_shared<LogicInput>(false, elm->outputSize());
 
+    case ElementType::AudioBox:
     case ElementType::Buzzer:
     case ElementType::Display14:
     case ElementType::Display7:
@@ -165,6 +169,7 @@ std::shared_ptr<LogicElement> ElementFactory::buildLogicElement(GraphicElement *
 
     case ElementType::And:         return std::make_shared<LogicAnd>(elm->inputSize());
     case ElementType::DFlipFlop:   return std::make_shared<LogicDFlipFlop>();
+    case ElementType::DLatch:      return std::make_shared<LogicDLatch>();
     case ElementType::Demux:       return std::make_shared<LogicDemux>();
     case ElementType::InputGnd:    return std::make_shared<LogicInput>(false);
     case ElementType::InputVcc:    return std::make_shared<LogicInput>(true);
@@ -176,12 +181,12 @@ std::shared_ptr<LogicElement> ElementFactory::buildLogicElement(GraphicElement *
     case ElementType::Not:         return std::make_shared<LogicNot>();
     case ElementType::Or:          return std::make_shared<LogicOr>(elm->inputSize());
     case ElementType::SRFlipFlop:  return std::make_shared<LogicSRFlipFlop>();
+    case ElementType::SRLatch:     return std::make_shared<LogicSRLatch>();
     case ElementType::TFlipFlop:   return std::make_shared<LogicTFlipFlop>();
+    case ElementType::TruthTable:  return std::make_shared<LogicTruthTable>(elm->inputSize(), elm->outputSize(), (dynamic_cast<TruthTable*>(elm))->key());
     case ElementType::Xnor:        return std::make_shared<LogicXnor>(elm->inputSize());
     case ElementType::Xor:         return std::make_shared<LogicXor>(elm->inputSize());
 
-    case ElementType::DLatch:      return std::make_shared<LogicDLatch>();
-
     case ElementType::Line:
     case ElementType::Text:        return std::make_shared<LogicNone>();
 
