@@ -1,9 +1,11 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "elementlabel.h"
 
 #include "elementfactory.h"
+#include "globalproperties.h"
+#include "serialization.h"
 
 #include <QDrag>
 #include <QFileInfo>
@@ -85,11 +87,11 @@ void ElementLabel::startDrag()
 #endif
     QByteArray itemData;
     QDataStream stream(&itemData, QIODevice::WriteOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    Serialization::writeHeaderPanda(stream);
     stream << offset << m_elementType << m_icFileName;
 
     auto *mimeData = new QMimeData();
-    mimeData->setData("wpanda/x-dnditemdata", itemData);
+    mimeData->setData("application/x-wiredpanda-dragdrop", itemData);
 
     auto *drag = new QDrag(parent());
     drag->setMimeData(mimeData);
@@ -107,11 +109,11 @@ QMimeData *ElementLabel::mimeData()
 #endif
     QByteArray itemData;
     QDataStream stream(&itemData, QIODevice::WriteOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    Serialization::writeHeaderPanda(stream);
     stream << offset << m_elementType << m_icFileName;
 
     auto *mimeData = new QMimeData();
-    mimeData->setData("wpanda/x-dnditemdata", itemData);
+    mimeData->setData("application/x-wiredpanda-dragdrop", itemData);
 
     return mimeData;
 }
