@@ -1,7 +1,6 @@
 #pragma once
 
 #include <QObject>
-#include <cstdint>
 
 class Enums
 {
@@ -13,8 +12,9 @@ public:
     enum class Status { Invalid = -1, Inactive = 0, Active = 1 };
     Q_ENUM(Status)
 
-    enum class ElementType {
+    enum class ElementType { // last element is 32
         And = 5,
+        AudioBox = 31,
         Buzzer = 26,
         Clock = 9,
         DFlipFlop = 17,
@@ -39,8 +39,10 @@ public:
         Not = 4,
         Or = 6,
         SRFlipFlop = 19,
+        SRLatch = 32,
         TFlipFlop = 20,
         Text = 28,
+        TruthTable = 30,
         Unknown = 0,
         Xnor = 11,
         Xor = 10,
@@ -59,6 +61,9 @@ public:
         Unknown = 0,
     };
     Q_ENUM(ElementGroup)
+
+    static ElementType nextElmType(ElementType type);
+    static ElementType prevElmType(ElementType type);
 };
 
 using Status = Enums::Status;
