@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "graphicelement.h"
@@ -18,12 +18,6 @@
 #include <QPixmap>
 #include <QStyleOptionGraphicsItem>
 #include <cmath>
-#include <iostream>
-
-namespace
-{
-    int id = qRegisterMetaType<GraphicElement>();
-}
 
 const int maximumValidInputSize = 256;
 
@@ -43,10 +37,10 @@ GraphicElement::GraphicElement(ElementType type, ElementGroup group, const QStri
         return;
     }
 
-    qCDebug(four) << tr("Setting flags of elements.");
+    qCDebug(four) << "Setting flags of elements.";
     setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable | QGraphicsItem::ItemSendsGeometryChanges);
 
-    qCDebug(four) << tr("Setting attributes.");
+    qCDebug(four) << "Setting attributes.";
     m_label->hide();
     QFont font("SansSerif");
     font.setBold(true);
@@ -58,14 +52,9 @@ GraphicElement::GraphicElement(ElementType type, ElementGroup group, const QStri
     setPortName(m_translatedName);
     setToolTip(m_translatedName);
 
-    qCDebug(four) << tr("Including input and output ports.");
-    for (int i = 0; i < minInputSize; ++i) {
-        addInputPort();
-    }
-
-    for (int i = 0; i < minOutputSize; ++i) {
-        addOutputPort();
-    }
+    qCDebug(four) << "Including input and output ports.";
+    setInputSize(minInputSize);
+    setOutputSize(minOutputSize);
 
     GraphicElement::updatePortsProperties();
     GraphicElement::updateTheme();
@@ -85,7 +74,7 @@ ElementGroup GraphicElement::elementGroup() const
 
 QPixmap GraphicElement::pixmap() const
 {
-    return m_pixmap ? *m_pixmap : QPixmap();
+    return m_pixmap;
 }
 
 void GraphicElement::setPixmap(const int index)
@@ -99,14 +88,10 @@ void GraphicElement::setPixmap(const QString &pixmapPath)
         return;
     }
 
-    if (!QPixmapCache::find(pixmapPath, m_pixmap.get())) {
-        if (!m_pixmap->load(pixmapPath)) {
-            m_pixmap->load(m_defaultSkins.constFirst());
-            qCDebug(zero) << tr("Problem loading pixmapPath: ") << pixmapPath;
-            throw Pandaception(tr("Couldn't load pixmap."));
-        }
-
-        QPixmapCache::insert(pixmapPath, *m_pixmap);
+    if (!m_pixmap.load(pixmapPath)) {
+        m_pixmap.load(m_defaultSkins.constFirst());
+        qCDebug(zero) << "Problem loading pixmapPath: " << pixmapPath;
+        throw Pandaception(tr("Couldn't load pixmap."));
     }
 
     setTransformOriginPoint(pixmapCenter());
@@ -137,7 +122,7 @@ QNEOutputPort *GraphicElement::outputPort(const int index)
 
 void GraphicElement::save(QDataStream &stream) const
 {
-    qCDebug(four) << tr("Saving element. Type: ") << objectName();
+    qCDebug(four) << "Saving element. Type: " << objectName();
 
     QMap<QString, QVariant> map;
     map.insert("pos", pos());
@@ -210,20 +195,20 @@ void GraphicElement::save(QDataStream &stream) const
 
     // -------------------------------------------
 
-    qCDebug(four) << tr("Finished saving element.");
+    qCDebug(four) << "Finished saving element.";
 }
 
 void GraphicElement::load(QDataStream &stream, QMap<quint64, QNEPort *> &portMap, const QVersionNumber version)
 {
-    qCDebug(four) << tr("Loading element. Type: ") << objectName();
+    qCDebug(four) << "Loading element. Type: " << objectName();
 
     (version < VERSION("4.1")) ? loadOldFormat(stream, portMap, version) : loadNewFormat(stream, portMap);
 
-    qCDebug(four) << tr("Updating port positions.");
+    qCDebug(four) << "Updating port positions.";
     updatePortsProperties();
     setRotation(m_angle);
 
-    qCDebug(four) << tr("Finished loading element.");
+    qCDebug(four) << "Finished loading element.";
 }
 
 void GraphicElement::loadOldFormat(QDataStream &stream, QMap<quint64, QNEPort *> &portMap, const QVersionNumber version)
@@ -295,7 +280,7 @@ void GraphicElement::loadNewFormat(QDataStream &stream, QMap<quint64, QNEPort *>
     QList<QMap<QString, QVariant>> inputMap; stream >> inputMap;
     int port = 0;
 
-    for (const auto &input : qAsConst(inputMap)) {
+    for (const auto &input : std::as_const(inputMap)) {
         const quint64 ptr = input.value("ptr").toULongLong();
         const QString name = input.value("name").toString();
 
@@ -311,7 +296,7 @@ void GraphicElement::loadNewFormat(QDataStream &stream, QMap<quint64, QNEPort *>
 
         portMap[ptr] = m_inputPorts.value(port);
 
-        port++;
+        ++port;
     }
 
     removeSurplusInputs(inputMap.size(), portMap);
@@ -321,7 +306,7 @@ void GraphicElement::loadNewFormat(QDataStream &stream, QMap<quint64, QNEPort *>
     QList<QMap<QString, QVariant>> outputMap; stream >> outputMap;
     port = 0;
 
-    for (const auto &output : qAsConst(outputMap)) {
+    for (const auto &output : std::as_const(outputMap)) {
         const quint64 ptr = output.value("ptr").toULongLong();
         const QString name = output.value("name").toString();
 
@@ -337,7 +322,7 @@ void GraphicElement::loadNewFormat(QDataStream &stream, QMap<quint64, QNEPort *>
 
         portMap[ptr] = m_outputPorts.value(port);
 
-        port++;
+        ++port;
     }
 
     removeSurplusOutputs(outputMap.size(), portMap);
@@ -347,17 +332,20 @@ void GraphicElement::loadNewFormat(QDataStream &stream, QMap<quint64, QNEPort *>
     QList<QMap<QString, QVariant>> skinsMap; stream >> skinsMap;
     int skin = 0;
 
-    for (const auto &skinName : qAsConst(skinsMap)) {
+    for (const auto &skinName : std::as_const(skinsMap)) {
         const QString name = skinName.value("skinName").toString();
 
         if (!name.startsWith(":/")) {
             m_alternativeSkins[skin] = name;
         }
 
-        skin++;
+        ++skin;
     }
 
-    m_usingDefaultSkin = (m_defaultSkins == m_alternativeSkins);
+    m_usingDefaultSkin = std::equal(
+        m_defaultSkins.begin(), m_defaultSkins.end(),
+        m_alternativeSkins.begin(), m_alternativeSkins.end()
+        );
 
     refresh();
 }
@@ -434,7 +422,7 @@ void GraphicElement::loadPriority(QDataStream &stream, const QVersionNumber vers
 
 void GraphicElement::loadInputPorts(QDataStream &stream, QMap<quint64, QNEPort *> &portMap)
 {
-    qCDebug(four) << tr("Loading input ports.");
+    qCDebug(four) << "Loading input ports.";
     quint64 inputSize; stream >> inputSize;
 
     if (inputSize > maximumValidInputSize) {
@@ -510,7 +498,7 @@ void GraphicElement::removePortFromMap(QNEPort *deletedPort, QMap<quint64, QNEPo
 
 void GraphicElement::loadOutputPorts(QDataStream &stream, QMap<quint64, QNEPort *> &portMap)
 {
-    qCDebug(four) << tr("Loading output ports.");
+    qCDebug(four) << "Loading output ports.";
     quint64 outputSize; stream >> outputSize;
 
     if (outputSize > maximumValidInputSize) {
@@ -557,7 +545,10 @@ void GraphicElement::loadPixmapSkinNames(QDataStream &stream, const QVersionNumb
             loadPixmapSkinName(stream, static_cast<int>(skin));
         }
 
-        m_usingDefaultSkin = (m_defaultSkins == m_alternativeSkins);
+        m_usingDefaultSkin = std::equal(
+            m_defaultSkins.begin(), m_defaultSkins.end(),
+            m_alternativeSkins.begin(), m_alternativeSkins.end()
+            );
 
         refresh();
     }
@@ -568,7 +559,7 @@ void GraphicElement::loadPixmapSkinName(QDataStream &stream, const int skin)
     QString name; stream >> name;
 
     if (skin >= m_alternativeSkins.size()) {
-        qCDebug(zero) << tr("Could not load some of the skins.");
+        qCDebug(zero) << "Could not load some of the skins.";
     }
 
     if (!name.startsWith(":/")) {
@@ -636,7 +627,7 @@ void GraphicElement::addPort(const QString &name, const bool isOutput, const int
         return;
     }
 
-    qCDebug(four) << tr("New port.");
+    qCDebug(four) << "New port.";
     QNEPort *port = nullptr;
 
     if (isOutput) {
@@ -683,13 +674,13 @@ void GraphicElement::setRotation(const qreal angle)
 
 void GraphicElement::rotatePorts(const qreal angle)
 {
-    for (auto *port : qAsConst(m_inputPorts)) {
+    for (auto *port : std::as_const(m_inputPorts)) {
         port->setTransformOriginPoint(mapToItem(port, pixmapCenter()));
         port->setRotation(angle);
         port->updateConnections();
     }
 
-    for (auto *port : qAsConst(m_outputPorts)) {
+    for (auto *port : std::as_const(m_outputPorts)) {
         port->setTransformOriginPoint(mapToItem(port, pixmapCenter()));
         port->setRotation(angle);
         port->updateConnections();
@@ -715,15 +706,15 @@ void GraphicElement::setSkin(const bool defaultSkin, const QString &fileName)
 
 void GraphicElement::updatePortsProperties()
 {
-    qCDebug(five) << tr("Updating port positions that belong to the IC.");
+    qCDebug(five) << "Updating port positions that belong to the IC.";
 
     const int step = GlobalProperties::gridSize / 2;
 
     if (!m_inputPorts.isEmpty()) {
         int y = 32 - (m_inputPorts.size() * step) + step;
 
-        for (auto *port : qAsConst(m_inputPorts)) {
-            qCDebug(five) << tr("Setting input at ") << 0 << tr(", ") << y;
+        for (auto *port : std::as_const(m_inputPorts)) {
+            qCDebug(five) << "Setting input at " << 0 << ", " << y;
 
             if (!isRotatable()) {
                 port->setRotation(0);
@@ -743,8 +734,8 @@ void GraphicElement::updatePortsProperties()
     if (!m_outputPorts.isEmpty()) {
         int y = 32 - (m_outputPorts.size() * step) + step;
 
-        for (auto *port : qAsConst(m_outputPorts)) {
-            qCDebug(five) << tr("Setting output at ") << 64 << tr(", ") << y;
+        for (auto *port : std::as_const(m_outputPorts)) {
+            qCDebug(five) << "Setting output at " << 64 << ", " << y;
 
             if (!isRotatable()) {
                 port->setRotation(0);
@@ -774,7 +765,7 @@ QVariant GraphicElement::itemChange(QGraphicsItem::GraphicsItemChange change, co
     }
 
     if (change == ItemPositionChange) {
-        qCDebug(four) << tr("Align to grid.");
+        qCDebug(four) << "Align to grid.";
         QPointF newPos = value.toPointF();
         const int gridSize = GlobalProperties::gridSize / 2;
         const int xV = qRound(newPos.x() / gridSize) * gridSize;
@@ -783,12 +774,12 @@ QVariant GraphicElement::itemChange(QGraphicsItem::GraphicsItemChange change, co
     }
 
     if ((change == ItemScenePositionHasChanged) || (change == ItemRotationHasChanged) || (change == ItemTransformHasChanged)) {
-        qCDebug(four) << tr("Moves wires.");
-        for (auto *port : qAsConst(m_outputPorts)) {
+        qCDebug(four) << "Moves wires.";
+        for (auto *port : std::as_const(m_outputPorts)) {
             port->updateConnections();
         }
 
-        for (auto *port : qAsConst(m_inputPorts)) {
+        for (auto *port : std::as_const(m_inputPorts)) {
             port->updateConnections();
         }
     }
@@ -803,9 +794,11 @@ QVariant GraphicElement::itemChange(QGraphicsItem::GraphicsItemChange change, co
 
 bool GraphicElement::sceneEvent(QEvent *event)
 {
-    if (auto mouseEvent = dynamic_cast<QGraphicsSceneMouseEvent *>(event)) {
-        if (mouseEvent->modifiers().testFlag(Qt::ControlModifier)) {
-            return true;
+    if (event->type() == QEvent::GraphicsSceneMousePress || event->type() == QEvent::GraphicsSceneMouseRelease) {
+        if (auto mouseEvent = dynamic_cast<QGraphicsSceneMouseEvent *>(event)) {
+            if (mouseEvent->modifiers().testFlag(Qt::ControlModifier)) {
+                return true;
+            }
         }
     }
 
@@ -864,6 +857,52 @@ QString GraphicElement::label() const
     return m_labelText;
 }
 
+QString GraphicElement::previousColor() const
+{
+    if (color() == "White") return "Purple";
+    if (color() == "Red") return "White";
+    if (color() == "Green") return "Red";
+    if (color() == "Blue") return "Green";
+    if (color() == "Purple") return "Blue";
+    return "White"; // Standard
+}
+
+QString GraphicElement::nextColor() const
+{
+    if (color() == "White") return "Red";
+    if (color() == "Red") return "Green";
+    if (color() == "Green") return "Blue";
+    if (color() == "Blue") return "Purple";
+    if (color() == "Purple") return "White";
+    return "White"; // Standard
+}
+
+QString GraphicElement::previousAudio() const
+{
+    if (audio() == "C6") return "C7";
+    if (audio() == "D6") return "C6";
+    if (audio() == "E6") return "D6";
+    if (audio() == "F6") return "E6";
+    if (audio() == "G6") return "F6";
+    if (audio() == "A7") return "G6";
+    if (audio() == "B7") return "A7";
+    if (audio() == "C7") return "B7";
+    return "C6";
+}
+
+QString GraphicElement::nextAudio() const
+{
+    if (audio() == "C6") return "D6";
+    if (audio() == "D6") return "E6";
+    if (audio() == "E6") return "F6";
+    if (audio() == "F6") return "G6";
+    if (audio() == "G6") return "A7";
+    if (audio() == "A7") return "B7";
+    if (audio() == "B7") return "C7";
+    if (audio() == "C7") return "C6";
+    return "C6";
+}
+
 void GraphicElement::updateTheme()
 {
     const ThemeAttributes theme = ThemeManager::attributes();
@@ -872,11 +911,11 @@ void GraphicElement::updateTheme()
     m_selectionBrush = theme.m_selectionBrush;
     m_selectionPen = theme.m_selectionPen;
 
-    for (auto *input : qAsConst(m_inputPorts)) {
+    for (auto *input : std::as_const(m_inputPorts)) {
         input->updateTheme();
     }
 
-    for (auto *output : qAsConst(m_outputPorts)) {
+    for (auto *output : std::as_const(m_outputPorts)) {
         output->updateTheme();
     }
 
@@ -885,12 +924,12 @@ void GraphicElement::updateTheme()
 
 bool GraphicElement::isValid()
 {
-    qCDebug(four) << tr("Checking if the element has the required signals to compute its value.");
+    qCDebug(four) << "Checking if the element has the required signals to compute its value.";
     const bool valid = std::all_of(m_inputPorts.cbegin(), m_inputPorts.cend(),
                                    [](auto *input) { return input->isValid(); });
 
     if (!valid) {
-        for (auto *output : qAsConst(m_outputPorts)) {
+        for (auto *output : std::as_const(m_outputPorts)) {
             for (auto *conn : output->connections()) {
                 conn->setStatus(Status::Invalid);
 
@@ -959,11 +998,41 @@ void GraphicElement::setHasFrequency(const bool hasFrequency)
     m_hasFrequency = hasFrequency;
 }
 
+bool GraphicElement::hasDelay() const
+{
+    return m_hasDelay;
+}
+
+void GraphicElement::setHasDelay(const bool hasDelay)
+{
+    m_hasDelay = hasDelay;
+}
+
 bool GraphicElement::hasLabel() const
 {
     return m_hasLabel;
 }
 
+bool GraphicElement::hasTruthTable() const
+{
+    return m_hasTruthTable;
+}
+
+void GraphicElement::setHasTruthTable(const bool hasTruthTable)
+{
+    m_hasTruthTable = hasTruthTable;
+}
+
+void GraphicElement::setHasAudioBox(const bool hasAudioBox)
+{
+    m_hasAudioBox = hasAudioBox;
+}
+
+bool GraphicElement::hasAudioBox() const
+{
+    return m_hasAudioBox;
+}
+
 bool GraphicElement::canChangeSkin() const
 {
     return m_canChangeSkin;
@@ -998,8 +1067,8 @@ int GraphicElement::inputSize() const
 void GraphicElement::setInputSize(const int size)
 {
     if ((size >= minInputSize()) && (size <= maxInputSize())) {
-        if (inputSize() < size) {
-            while (inputSize() < size) {
+        if (size > inputSize()) {
+            while (size > inputSize()) {
                 addInputPort();
             }
         } else {
@@ -1024,8 +1093,8 @@ int GraphicElement::priority() const
 void GraphicElement::setOutputSize(const int size)
 {
     if ((size >= minOutputSize()) && (size <= maxOutputSize())) {
-        if (outputSize() < size) {
-            for (int port = outputSize(); port < size; ++port) {
+        if (size > outputSize()) {
+            while (size > outputSize()) {
                 addOutputPort();
             }
         } else {
@@ -1047,6 +1116,16 @@ void GraphicElement::setFrequency(const float freq)
     Q_UNUSED(freq)
 }
 
+float GraphicElement::delay() const
+{
+    return 0.0;
+}
+
+void GraphicElement::setDelay(const float delay)
+{
+    Q_UNUSED(delay)
+}
+
 void GraphicElement::setMinOutputSize(const int minOutputSize)
 {
     m_minOutputSize = minOutputSize;
@@ -1086,15 +1165,15 @@ void GraphicElement::highlight(const bool isSelected)
 {
     QVector<QNEPort *> ports;
 
-    for (auto *port : qAsConst(m_inputPorts)) {
+    for (auto *port : std::as_const(m_inputPorts)) {
         ports << port;
     }
 
-    for (auto *port : qAsConst(m_outputPorts)) {
+    for (auto *port : std::as_const(m_outputPorts)) {
         ports << port;
     }
 
-    for (auto *port : qAsConst(ports)) {
+    for (auto *port : std::as_const(ports)) {
         for (auto *connection : port->connections()) {
             if (connection->highLight() == isSelected) {
                 continue;
@@ -1115,7 +1194,7 @@ void GraphicElement::retranslate()
 
 QDataStream &operator<<(QDataStream &stream, const GraphicElement *item)
 {
-    qCDebug(four) << QObject::tr("Writing element.");
+    qCDebug(four) << "Writing element.";
     const auto *elm = qgraphicsitem_cast<const GraphicElement *>(item);
     stream << GraphicElement::Type;
     stream << elm->elementType();
