@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "graphicsview.h"
@@ -29,7 +29,7 @@ bool GraphicsView::canZoomIn() const
 
 bool GraphicsView::canZoomOut() const
 {
-    return m_zoomLevel > -3;
+    return m_zoomLevel > -9;
 }
 
 void GraphicsView::mousePressEvent(QMouseEvent *event)
@@ -99,15 +99,30 @@ void GraphicsView::keyReleaseEvent(QKeyEvent *event)
 
 void GraphicsView::wheelEvent(QWheelEvent *event)
 {
-    if (QApplication::keyboardModifiers() == Qt::ControlModifier) {
-        double deltaY = event->angleDelta().y();
-        (deltaY > 0) ? m_zoomLevel++ : m_zoomLevel--;
-        double factor = (deltaY > 0) ? 1.25 : 0.8;
-        scale(factor, factor);
-        return;
+    const int zoomDirection = event->angleDelta().y();
+
+    if (zoomDirection > 0 && canZoomIn()) {
+        if (m_redirectZoom) {
+            emit scaleIn();
+        } else {
+            zoomIn();
+        }
+    } else if (zoomDirection < 0 && canZoomOut()) {
+        if (m_redirectZoom) {
+            emit scaleOut();
+        } else {
+            zoomOut();
+        }
     }
 
-    QGraphicsView::wheelEvent(event);
+    centerOn(QCursor::pos());
+
+    event->accept();
+}
+
+void GraphicsView::setRedirectZoom(const bool value)
+{
+    m_redirectZoom = value;
 }
 
 void GraphicsView::setFastMode(const bool fastMode)
@@ -120,7 +135,7 @@ void GraphicsView::setFastMode(const bool fastMode)
 void GraphicsView::zoomIn()
 {
     scale(1.25, 1.25);
-    m_zoomLevel++;
+    ++m_zoomLevel;
     emit zoomChanged();
 }
 
