@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
@@ -16,10 +16,13 @@ public:
     bool canZoomOut() const;
     void resetZoom();
     void setFastMode(const bool fastMode);
+    void setRedirectZoom(const bool value);
     void zoomIn();
     void zoomOut();
 
 signals:
+    void scaleIn();
+    void scaleOut();
     void zoomChanged();
 
 protected:
@@ -32,6 +35,7 @@ protected:
 
 private:
     bool m_pan = false;
+    bool m_redirectZoom = false;
     bool m_space = false;
     int m_panStartX = 0;
     int m_panStartY = 0;
