@@ -1,14 +1,14 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
-#include <QSet>
 #include <QVector>
 
 class LogicElement;
 
-struct InputPair {
+struct InputPair
+{
     LogicElement *logic = nullptr;
     int port = 0;
 };
@@ -19,12 +19,12 @@ class LogicElement
 public:
     explicit LogicElement(const int inputSize, const int outputSize);
     virtual ~LogicElement() = default;
-    bool operator>(const LogicElement &other) const;
 
     bool inputValue(const int index = 0) const;
     bool isValid() const;
     bool outputValue(const int index = 0) const;
     int calculatePriority();
+    int priority() const;
     virtual void updateLogic() = 0;
     void clearSucessors();
     void connectPredecessor(const int index, LogicElement *logic, const int port);
@@ -40,7 +40,7 @@ protected:
 private:
     Q_DISABLE_COPY(LogicElement)
 
-    QSet<LogicElement *> m_successors;
+    QVector<LogicElement *> m_successors;
     QVector<InputPair> m_inputPairs;
     QVector<bool> m_outputValues;
     bool m_beingVisited = false;
