@@ -1,10 +1,11 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "application.h"
 #include "common.h"
 #include "globalproperties.h"
 #include "mainwindow.h"
+#include "registertypes.h"
 
 #include <QCommandLineParser>
 #include <QMessageBox>
@@ -13,8 +14,29 @@
 #include <windows.h>
 #endif
 
+#ifdef HAVE_SENTRY
+#include "../thirdparty/sentry/include/sentry.h"
+#include <QScopeGuard>
+#endif
+
 int main(int argc, char *argv[])
 {
+#ifdef HAVE_SENTRY
+    sentry_options_t *options = sentry_options_new();
+    sentry_options_set_dsn(options, "https://719a4881adf6e678b198bf9aad6b4500@o4508704323469312.ingest.us.sentry.io/4508704326352896");
+    // This is also the default-path. For further information and recommendations:
+    // https://docs.sentry.io/platforms/native/configuration/options/#database-path
+    sentry_options_set_database_path(options, ".sentry-native");
+    sentry_options_set_release(options, "wiredpanda@" APP_VERSION);
+    sentry_options_set_debug(options, 0);
+    sentry_init(options);
+
+    // Make sure everything flushes
+    auto sentryClose = qScopeGuard([] { sentry_close(); });
+#endif
+
+    registerTypes();
+
     Comment::setVerbosity(-1);
 
 #ifdef Q_OS_WIN
@@ -32,7 +54,7 @@ int main(int argc, char *argv[])
 
     Application app(argc, argv);
     app.setOrganizationName("GIBIS-UNIFESP");
-    app.setApplicationName("WiRedPanda");
+    app.setApplicationName("wiRedPanda");
     app.setApplicationVersion(APP_VERSION);
     app.setStyle("Fusion");
     app.setWindowIcon(QIcon(":/toolbar/wpanda.svg"));
@@ -52,19 +74,19 @@ int main(int argc, char *argv[])
 
         QCommandLineOption arduinoFileOption(
             {"a", "arduino-file"},
-            QCoreApplication::translate("main", "Export circuit to <arduino-file>"),
+            QCoreApplication::translate("main", "Exports circuit to <arduino-file>"),
             QCoreApplication::translate("main", "arduino file"));
         parser.addOption(arduinoFileOption);
 
         QCommandLineOption waveformFileOption(
             {"w", "waveform"},
-            QCoreApplication::translate("main", "Export circuit to waveform text file"),
+            QCoreApplication::translate("main", "Exports circuit to waveform text file"),
             QCoreApplication::translate("main", "waveform input text file"));
         parser.addOption(waveformFileOption);
 
         QCommandLineOption terminalFileOption(
             {"c", "terminal"},
-            QCoreApplication::translate("main", "Export circuit to waveform text file, reading input from terminal"));
+            QCoreApplication::translate("main", "Exports circuit to waveform text file, reading input from terminal"));
         parser.addOption(terminalFileOption);
 
         parser.process(app);
