@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "mainwindow.h"
@@ -35,6 +35,10 @@
 #include <QTemporaryFile>
 #include <QTranslator>
 
+#ifdef Q_OS_WASM
+#include <emscripten/emscripten.h>
+#endif
+
 #if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
 #define SKIPEMPTYPARTS QString::SkipEmptyParts
 #else
@@ -45,22 +49,22 @@ MainWindow::MainWindow(const QString &fileName, QWidget *parent)
     : QMainWindow(parent)
     , m_ui(new Ui::MainWindow)
 {
-    qCDebug(zero) << tr("WiRedPanda Version = ") << APP_VERSION << tr(" OR ") << GlobalProperties::version;
+    qCDebug(zero) << "wiRedPanda Version = " APP_VERSION " OR" << GlobalProperties::version;
     m_ui->setupUi(this);
 
-    qCDebug(zero) << tr("Settings fileName: ") << Settings::fileName();
+    qCDebug(zero) << "Settings fileName: " << Settings::fileName();
     loadTranslation(Settings::value("language").toString());
 
     connect(m_ui->tab, &QTabWidget::currentChanged,    this, &MainWindow::tabChanged);
     connect(m_ui->tab, &QTabWidget::tabCloseRequested, this, &MainWindow::closeTab);
 
-    qCDebug(zero) << tr("Restoring geometry and setting zoom controls.");
+    qCDebug(zero) << "Restoring geometry and setting zoom controls.";
     restoreGeometry(Settings::value("MainWindow/geometry").toByteArray());
     restoreState(Settings::value("MainWindow/windowState").toByteArray());
     m_ui->splitter->restoreGeometry(Settings::value("MainWindow/splitter/geometry").toByteArray());
     m_ui->splitter->restoreState(Settings::value("MainWindow/splitter/state").toByteArray());
 
-    qCDebug(zero) << tr("Preparing theme and UI modes.");
+    qCDebug(zero) << "Preparing theme and UI modes.";
     auto *themeGroup = new QActionGroup(this);
     const auto actions = m_ui->menuTheme->actions();
 
@@ -75,43 +79,42 @@ MainWindow::MainWindow(const QString &fileName, QWidget *parent)
     m_ui->actionLabelsUnderIcons->setChecked(Settings::value("labelsUnderIcons").toBool());
     m_ui->mainToolBar->setToolButtonStyle(Settings::value("labelsUnderIcons").toBool() ? Qt::ToolButtonTextUnderIcon : Qt::ToolButtonIconOnly);
 
-    qCDebug(zero) << tr("Setting left side menus.");
-    auto *shortcut = new QShortcut(QKeySequence(Qt::CTRL | Qt::Key_F), this);
-    connect(shortcut, &QShortcut::activated, m_ui->lineEditSearch, qOverload<>(&QWidget::setFocus));
+    qCDebug(zero) << "Setting left side menus.";
     populateLeftMenu();
     m_ui->tabElements->setTabIcon(0, QIcon(":/input/buttonOff.svg"));
     m_ui->tabElements->setTabIcon(1, QIcon(":/basic/xor.svg"));
-    m_ui->tabElements->setTabIcon(2, QIcon(DFlipFlop::pixmapPath()));
-    m_ui->tabElements->setTabIcon(3, QIcon(":/basic/ic-panda.svg"));
-    m_ui->tabElements->setTabIcon(4, QIcon(":/misc/text.png"));
-    m_ui->tabElements->setTabEnabled(5, false);
+    m_ui->tabElements->setTabIcon(2, QIcon(":/basic/truthtable-rotated.svg"));
+    m_ui->tabElements->setTabIcon(3, QIcon(DFlipFlop::pixmapPath()));
+    m_ui->tabElements->setTabIcon(4, QIcon(":/basic/ic-panda.svg"));
+    m_ui->tabElements->setTabIcon(5, QIcon(":/misc/text.png"));
+    m_ui->tabElements->setTabEnabled(6, false);
 
-    qCDebug(zero) << tr("Loading recent file list.");
+    qCDebug(zero) << "Loading recent file list.";
     m_recentFiles = new RecentFiles(this);
     connect(this, &MainWindow::addRecentFile, m_recentFiles, &RecentFiles::addRecentFile);
     createRecentFileActions();
     connect(m_recentFiles, &RecentFiles::recentFilesUpdated, this, &MainWindow::updateRecentFileActions);
 
-    qCDebug(zero) << tr("Checking playing simulation.");
+    qCDebug(zero) << "Checking playing simulation.";
     m_ui->actionPlay->setChecked(true);
 
-    qCDebug(zero) << tr("Window title.");
-    setWindowTitle("WiRedPanda " + QString(APP_VERSION));
+    qCDebug(zero) << "Window title.";
+    setWindowTitle("wiRedPanda " APP_VERSION);
 
-    qCDebug(zero) << tr("Building a new tab.");
+    qCDebug(zero) << "Building a new tab.";
     createNewTab();
 
-    qCDebug(zero) << tr("Opening file if not empty.");
+    qCDebug(zero) << "Opening file if not empty.";
     if (!fileName.isEmpty()) {
         loadPandaFile(fileName);
     }
 
-    qCDebug(zero) << tr("Disabling Arduino export.");
+    qCDebug(zero) << "Disabling Arduino export.";
     m_ui->actionExportToArduino->setEnabled(false);
 
     QPixmapCache::setCacheLimit(100'000);
 
-    qCDebug(zero) << tr("Adding examples to menu");
+    qCDebug(zero) << "Adding examples to menu";
     QDir examplesDir("examples");
 
     if (examplesDir.exists()) {
@@ -123,7 +126,8 @@ MainWindow::MainWindow(const QString &fileName, QWidget *parent)
             connect(action, &QAction::triggered, this, [this] {
                 if (auto *action = qobject_cast<QAction *>(sender())) {
                     loadPandaFile("examples/" + action->text());
-                }});
+                }
+            });
 
             m_ui->menuExamples->addAction(action);
         }
@@ -133,7 +137,24 @@ MainWindow::MainWindow(const QString &fileName, QWidget *parent)
         m_ui->menuExamples->menuAction()->setVisible(false);
     }
 
-    qCDebug(zero) << tr("Setting connections");
+    // Shortcuts
+    auto *searchShortcut = new QShortcut(QKeySequence(Qt::CTRL | Qt::Key_F), this);
+    auto *prevMainPropShortcut = new QShortcut(QKeySequence("["), this);
+    auto *nextMainPropShortcut = new QShortcut(QKeySequence("]"), this);
+    auto *prevSecndPropShortcut = new QShortcut(QKeySequence("{"), this);
+    auto *nextSecndPropShortcut = new QShortcut(QKeySequence("}"), this);
+    auto *changePrevElmShortcut = new QShortcut(QKeySequence("<"), this);
+    auto *changeNextElmShortcut = new QShortcut(QKeySequence(">"), this);
+
+    connect(searchShortcut,               &QShortcut::activated,      m_ui->lineEditSearch,  qOverload<>(&QWidget::setFocus));
+    connect(prevMainPropShortcut,         &QShortcut::activated,      m_currentTab->scene(), &Scene::prevMainPropShortcut);
+    connect(nextMainPropShortcut,         &QShortcut::activated,      m_currentTab->scene(), &Scene::nextMainPropShortcut);
+    connect(prevSecndPropShortcut,        &QShortcut::activated,      m_currentTab->scene(), &Scene::prevSecndPropShortcut);
+    connect(nextSecndPropShortcut,        &QShortcut::activated,      m_currentTab->scene(), &Scene::nextSecndPropShortcut);
+    connect(changePrevElmShortcut,        &QShortcut::activated,      m_currentTab->scene(), &Scene::prevElm);
+    connect(changeNextElmShortcut,        &QShortcut::activated,      m_currentTab->scene(), &Scene::nextElm);
+
+    qCDebug(zero) << "Setting connections";
     connect(m_ui->actionAbout,            &QAction::triggered,        this,                &MainWindow::on_actionAbout_triggered);
     connect(m_ui->actionAboutQt,          &QAction::triggered,        this,                &MainWindow::on_actionAboutQt_triggered);
     connect(m_ui->actionAboutThisVersion, &QAction::triggered,        this,                &MainWindow::aboutThisVersion);
@@ -165,6 +186,8 @@ MainWindow::MainWindow(const QString &fileName, QWidget *parent)
     connect(m_ui->actionSave,             &QAction::triggered,        this,                &MainWindow::on_actionSave_triggered);
     connect(m_ui->actionSaveAs,           &QAction::triggered,        this,                &MainWindow::on_actionSaveAs_triggered);
     connect(m_ui->actionSelectAll,        &QAction::triggered,        this,                &MainWindow::on_actionSelectAll_triggered);
+    connect(m_ui->actionShortcutsAndTips, &QAction::triggered,        this,                &MainWindow::on_actionShortcuts_and_Tips_triggered);
+    connect(m_ui->actionSpanish,          &QAction::triggered,        this,                &MainWindow::on_actionSpanish_triggered);
     connect(m_ui->actionWaveform,         &QAction::triggered,        this,                &MainWindow::on_actionWaveform_triggered);
     connect(m_ui->actionWires,            &QAction::triggered,        this,                &MainWindow::on_actionWires_triggered);
     connect(m_ui->actionZoomIn,           &QAction::triggered,        this,                &MainWindow::on_actionZoomIn_triggered);
@@ -185,13 +208,13 @@ void MainWindow::loadAutosaveFiles()
 {
     QStringList autosaves(Settings::value("autosaveFile").toStringList());
 
-    qCDebug(zero) << tr("All autosave files: ") << autosaves;
+    qCDebug(zero) << "All autosave files: " << autosaves;
 
     for (auto it = autosaves.begin(); it != autosaves.end();) {
         QFile file(*it);
 
         if (!file.exists()) {
-            qCDebug(zero) << tr("Removing from config the autosave file that does not exist.");
+            qCDebug(zero) << "Removing from config the autosave file that does not exist.";
             it = autosaves.erase(it);
             continue;
         }
@@ -200,7 +223,7 @@ void MainWindow::loadAutosaveFiles()
             loadPandaFile(*it);
         } catch (const std::exception &e) {
             QMessageBox::critical(nullptr, tr("Error!"), e.what());
-            qCDebug(zero) << tr("Removing autosave file that is corrupted.");
+            qCDebug(zero) << "Removing autosave file that is corrupted.";
             it = autosaves.erase(it);
             continue;
         }
@@ -215,7 +238,7 @@ void MainWindow::loadAutosaveFiles()
 
 void MainWindow::createNewTab()
 {
-    qCDebug(zero) << tr("Creating new workspace.");
+    qCDebug(zero) << "Creating new workspace.";
     auto *workspace = new WorkSpace(this);
 
     connect(workspace, &WorkSpace::fileChanged, this, &MainWindow::setCurrentFile);
@@ -223,13 +246,13 @@ void MainWindow::createNewTab()
     workspace->view()->setFastMode(m_ui->actionFastMode->isChecked());
     workspace->scene()->updateTheme();
 
-    qCDebug(zero) << tr("Adding tab. #tabs: ") << m_ui->tab->count() << tr(", current tab: ") << m_tabIndex;
+    qCDebug(zero) << "Adding tab. #tabs: " << m_ui->tab->count() << ", current tab: " << m_tabIndex;
     m_ui->tab->addTab(workspace, tr("New Project"));
 
-    qCDebug(zero) << tr("Selecting the newly created tab.");
+    qCDebug(zero) << "Selecting the newly created tab.";
     m_ui->tab->setCurrentIndex(m_ui->tab->count() - 1);
 
-    qCDebug(zero) << tr("Finished #tabs: ") << m_ui->tab->count() << tr(", current tab: ") << m_tabIndex;
+    qCDebug(zero) << "Finished #tabs: " << m_ui->tab->count() << ", current tab: " << m_tabIndex;
 }
 
 void MainWindow::removeUndoRedoMenu()
@@ -281,13 +304,13 @@ void MainWindow::show()
         aboutThisVersion();
     }
 
-    qCDebug(zero) << tr("Checking for autosave file recovery.");
+    qCDebug(zero) << "Checking for autosave file recovery.";
     loadAutosaveFiles();
 }
 
 void MainWindow::aboutThisVersion()
 {
-    qCDebug(zero) << tr("'hideV4Warning' message box.");
+    qCDebug(zero) << "'hideV4Warning' message box.";
 
     auto *checkBox = new QCheckBox(tr("Don't show this again."));
 
@@ -295,22 +318,25 @@ void MainWindow::aboutThisVersion()
     msgBox.setParent(this);
     msgBox.setStandardButtons(QMessageBox::Ok);
     msgBox.setIcon(QMessageBox::Icon::Information);
-    msgBox.setWindowTitle("WiRedPanda " + QString(APP_VERSION));
+    msgBox.setWindowTitle("wiRedPanda " APP_VERSION);
     msgBox.setText(
-        tr("This version is not 100% compatible with previous versions of WiRedPanda.\n"
+        tr("wiRedPanda version >= 4.0 is not 100% compatible with previous versions.\n"
            "To open old version projects containing ICs (or boxes), skins, and/or "
            "beWavedDolphin simulations, their files must be moved to the same directory "
            "as the main project file.\n"
-           "WiRedPanda %1 will automatically list all other .panda files located "
+           "wiRedPanda " APP_VERSION " will automatically list all other .panda files located "
            "in the same directory of the current project as ICs in the editor tab.\n"
            "You have to save new projects before accessing ICs and skins, or running "
-           "beWavedDolphin simulations.")
-            .arg(APP_VERSION));
+           "beWavedDolphin simulations."));
     msgBox.setWindowModality(Qt::WindowModal);
     msgBox.setDefaultButton(QMessageBox::Ok);
     msgBox.setCheckBox(checkBox);
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 7, 0)
     connect(checkBox, &QCheckBox::stateChanged, this, [](int state) {
+#else
+    connect(checkBox, &QCheckBox::checkStateChanged, this, [](int state) {
+#endif
         if (static_cast<Qt::CheckState>(state) == Qt::CheckState::Checked) {
             Settings::setValue("hideV4Warning", "true");
         } else {
@@ -380,7 +406,7 @@ void MainWindow::on_actionRotateLeft_triggered()
 void MainWindow::loadPandaFile(const QString &fileName)
 {
     createNewTab();
-    qCDebug(zero) << tr("Loading in editor.");
+    qCDebug(zero) << "Loading in editor.";
     m_currentTab->load(fileName);
     updateICList();
     m_ui->statusBar->showMessage(tr("File loaded successfully."), 4000);
@@ -388,6 +414,19 @@ void MainWindow::loadPandaFile(const QString &fileName)
 
 void MainWindow::on_actionOpen_triggered()
 {
+#ifdef Q_OS_WASM
+    auto fileContentReady = [this](const QString &fileName, const QByteArray &fileContent) {
+        if (!fileName.isEmpty()) {
+            // Write file content to a temporary file
+            QFile file(fileName);
+            file.open(QIODevice::WriteOnly);
+            file.write(fileContent);
+            file.close();
+            loadPandaFile(fileName);
+        }
+    };
+    QFileDialog::getOpenFileContent("Panda files (*.panda)", fileContentReady);
+#else
     const QString path = m_currentFile.exists() ? "" : "./examples";
     const QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"), path, tr("Panda files (*.panda)"));
 
@@ -396,6 +435,7 @@ void MainWindow::on_actionOpen_triggered()
     }
 
     loadPandaFile(fileName);
+#endif
 }
 
 void MainWindow::on_actionSave_triggered()
@@ -444,8 +484,8 @@ void MainWindow::on_actionAbout_triggered()
 {
     QMessageBox::about(
         this,
-        "WiRedPanda",
-        tr("<p>WiRedPanda is a software developed by the students of the Federal University of São Paulo."
+        "wiRedPanda",
+        tr("<p>wiRedPanda is a software developed by the students of the Federal University of São Paulo."
            " This project was created in order to help students learn about logic circuits.</p>"
            "<p>Software version: %1</p>"
            "<p><strong>Creators:</strong></p>"
@@ -455,12 +495,40 @@ void MainWindow::on_actionAbout_triggered()
            "<li> Rodrigo Torres </li>"
            "<li> Prof. Fábio Cappabianco, Ph.D. </li>"
            "</ul>"
-           "<p> WiRedPanda is currently maintained by Prof. Fábio Cappabianco, Ph.D. and Vinícius R. Miguel.</p>"
+           "<p> wiRedPanda is currently maintained by Prof. Fábio Cappabianco, Ph.D., João Pedro M. Oliveira, Matheus R. Esteves e Maycon A. Santana.</p>"
            "<p> Please file a report at our GitHub page if bugs are found or if you wish for a new functionality to be implemented.</p>"
            "<p><a href=\"http://gibis-unifesp.github.io/wiRedPanda/\">Visit our website!</a></p>")
             .arg(QApplication::applicationVersion()));
 }
 
+void MainWindow::on_actionShortcuts_and_Tips_triggered()
+{
+    QMessageBox::information(this,
+        tr("Shortcuts and Tips"),
+        tr("<h1>Canvas Shortcuts</h1>"
+           "<ul style=\"list-style:none;\">"
+           "<li> Ctrl+= : Zoom in </li>"
+           "<li> Ctrl+- : Zoom out </li>"
+           "<li> Ctrl+1 : Hide/Show wires </li>"
+           "<li> Ctrl+2 : Hide/Show gates </li>"
+           "<li> Ctrl+F : Search elements </li>"
+           "<li> Ctrl+W : Open beWaveDolphin </li>"
+           "<li> Ctrl+S : Save project </li>"
+           "<li> Ctrl+Q : Exit wiRedPanda </li>"
+           "<li> F5 : Start/Pause simulation </li>"
+           "<li> [ : Previous primary element property </li>"
+           "<li> ] : Next primary element property </li>"
+           "<li> { : Previous secondary element property </li>"
+           "<li> } : Next secondary element property </li>"
+           "<li> &lt; : Morph to previous element </li>"
+           "<li> &gt; : Morph to next element </li>"
+           "</ul>"
+
+           "<h1>General Tips</h1>"
+           "<p>Double click in a wire to create a node</p>"
+        ));
+}
+
 void MainWindow::on_actionAboutQt_triggered()
 {
     QMessageBox::aboutQt(this);
@@ -479,6 +547,7 @@ bool MainWindow::closeFiles()
 void MainWindow::closeEvent(QCloseEvent *event)
 {
     bool closeWindow = false;
+
     if (!hasModifiedFiles()) {
         auto reply =
             QMessageBox::question(
@@ -546,6 +615,10 @@ QDir MainWindow::currentDir() const
 
 void MainWindow::setCurrentFile(const QFileInfo &fileInfo)
 {
+    if (!m_currentTab) {
+        return;
+    }
+
     m_currentFile = fileInfo;
 
     QString text = fileInfo.exists() ? fileInfo.fileName() : tr("New Project");
@@ -555,8 +628,9 @@ void MainWindow::setCurrentFile(const QFileInfo &fileInfo)
     }
 
     m_ui->tab->setTabText(m_tabIndex, text);
+    m_ui->tab->setTabToolTip(m_tabIndex, fileInfo.absoluteFilePath());
 
-    qCDebug(zero) << tr("Adding file to recent files.");
+    qCDebug(zero) << "Adding file to recent files.";
     emit addRecentFile(fileInfo.absoluteFilePath());
 }
 
@@ -584,7 +658,7 @@ void MainWindow::updateICList()
     }
 
     if (m_currentFile.exists()) {
-        qCDebug(zero) << tr("Show files.");
+        qCDebug(zero) << "Show files.";
         QDir directory(m_currentFile.absoluteDir());
         QStringList files = directory.entryList({"*.panda", "*.PANDA"}, QDir::Files);
         files.removeAll(m_currentFile.fileName());
@@ -595,8 +669,8 @@ void MainWindow::updateICList()
             }
         }
 
-        qCDebug(zero) << tr("Files: ") << files.join(", ");
-        for (const QString &file : qAsConst(files)) {
+        qCDebug(zero) << "Files: " << files.join(", ");
+        for (const QString &file : std::as_const(files)) {
             QPixmap pixmap(":/basic/ic-panda.svg");
 
             auto *item = new ElementLabel(pixmap, ElementType::IC, file, this);
@@ -612,10 +686,10 @@ void MainWindow::updateICList()
 
 bool MainWindow::closeTab(const int tabIndex)
 {
-    qCDebug(zero) << tr("Closing tab ") << tabIndex + 1 << tr(", #tabs: ") << m_ui->tab->count();
+    qCDebug(zero) << "Closing tab " << tabIndex + 1 << ", #tabs: " << m_ui->tab->count();
     m_ui->tab->setCurrentIndex(tabIndex);
 
-    qCDebug(zero) << tr("Checking if needs to save file.");
+    qCDebug(zero) << "Checking if needs to save file.";
 
     if (!m_currentTab->scene()->undoStack()->isClean()) {
         const int selectedButton = confirmSave(false);
@@ -637,11 +711,11 @@ bool MainWindow::closeTab(const int tabIndex)
         }
     }
 
-    qCDebug(zero) << tr("Deleting tab.");
+    qCDebug(zero) << "Deleting tab.";
     m_currentTab->deleteLater();
     m_ui->tab->removeTab(tabIndex);
 
-    qCDebug(zero) << tr("Closed tab ") << tabIndex << tr(", #tabs: ") << m_ui->tab->count() << tr(", current tab: ") << m_tabIndex;
+    qCDebug(zero) << "Closed tab " << tabIndex << ", #tabs: " << m_ui->tab->count() << ", current tab: " << m_tabIndex;
 
     return true;
 }
@@ -654,13 +728,13 @@ void MainWindow::disconnectTab()
 
     m_ui->elementEditor->setScene(nullptr);
 
-    qCDebug(zero) << tr("Stopping simulation.");
+    qCDebug(zero) << "Stopping simulation.";
     m_currentTab->simulation()->stop();
 
-    qCDebug(zero) << tr("Disconnecting zoom from UI.");
+    qCDebug(zero) << "Disconnecting zoom from UI.";
     disconnect(m_currentTab->view(), &GraphicsView::zoomChanged, this, &MainWindow::zoomChanged);
 
-    qCDebug(zero) << tr("Removing undo and redo actions from UI menu.");
+    qCDebug(zero) << "Removing undo and redo actions from UI menu.";
     removeUndoRedoMenu();
 
     disconnect(m_ui->actionClearSelection, &QAction::triggered,         m_currentTab->scene(), &Scene::clearSelection);
@@ -673,15 +747,15 @@ void MainWindow::disconnectTab()
 
 void MainWindow::connectTab()
 {
-    qCDebug(zero) << tr("Connecting undo and redo functions to UI menu.");
+    qCDebug(zero) << "Connecting undo and redo functions to UI menu.";
     addUndoRedoMenu();
 
-    qCDebug(zero) << tr("Setting Panda file info.");
+    qCDebug(zero) << "Setting Panda file info.";
     m_currentFile = m_currentTab->fileInfo();
 
     updateICList();
 
-    qCDebug(zero) << tr("Connecting current tab to element editor menu in UI.");
+    qCDebug(zero) << "Connecting current tab to element editor menu in UI.";
     m_ui->elementEditor->setScene(m_currentTab->scene());
 
     connect(m_currentTab->view(),       &GraphicsView::zoomChanged, this,                  &MainWindow::zoomChanged);
@@ -692,7 +766,7 @@ void MainWindow::connectTab()
     connect(m_ui->actionPaste,          &QAction::triggered,        m_currentTab->scene(), &Scene::pasteAction);
 
     if (m_ui->actionPlay->isChecked()) {
-        qCDebug(zero) << tr("Restarting simulation.");
+        qCDebug(zero) << "Restarting simulation.";
         m_currentTab->simulation()->start();
         m_currentTab->scene()->clearSelection();
     }
@@ -714,9 +788,9 @@ void MainWindow::tabChanged(const int newTabIndex)
     }
 
     m_currentTab = qobject_cast<WorkSpace *>(m_ui->tab->currentWidget());
-    qCDebug(zero) << tr("Selecting tab: ") << newTabIndex;
+    qCDebug(zero) << "Selecting tab: " << newTabIndex;
     connectTab();
-    qCDebug(zero) << tr("New tab selected. Dolphin fileName: ") << m_currentTab->dolphinFileName();
+    qCDebug(zero) << "New tab selected. Dolphin fileName: " << m_currentTab->dolphinFileName();
 }
 
 void MainWindow::on_lineEditSearch_textChanged(const QString &text)
@@ -765,7 +839,7 @@ void MainWindow::on_lineEditSearch_textChanged(const QString &text)
             item->setHidden(true);
         }
 
-        for (auto *item : qAsConst(searchResults)) {
+        for (auto *item : std::as_const(searchResults)) {
             item->setVisible(true);
         }
     }
@@ -826,7 +900,7 @@ void MainWindow::exportToArduino(QString fileName)
     auto elements = m_currentTab->scene()->elements();
 
     if (elements.isEmpty()) {
-        throw Pandaception(tr("The panda file is empty."));
+        throw Pandaception(tr("The .panda file is empty."));
     }
 
     SimulationBlocker simulationBlocker(m_currentTab->simulation());
@@ -841,7 +915,7 @@ void MainWindow::exportToArduino(QString fileName)
     arduino.generate();
     m_ui->statusBar->showMessage(tr("Arduino code successfully generated."), 4000);
 
-    qCDebug(zero) << tr("Arduino code successfully generated.");
+    qCDebug(zero) << "Arduino code successfully generated.";
 }
 
 void MainWindow::exportToWaveFormFile(const QString &fileName)
@@ -1106,11 +1180,16 @@ void MainWindow::loadTranslation(const QString &language)
         qtFile = ":/translations/qt_pt_BR.qm";
     }
 
+    if (language == "es") {
+        pandaFile = ":/translations/wpanda_es.qm";
+        qtFile = ":/translations/qt_es.qm";
+    }
+
     if (!pandaFile.isEmpty()) {
         m_pandaTranslator = new QTranslator(this);
 
         if (!m_pandaTranslator->load(pandaFile) || !qApp->installTranslator(m_pandaTranslator)) {
-            throw Pandaception(tr("Error loading WiRedPanda translation!"));
+            throw Pandaception(tr("Error loading wiRedPanda translation!"));
         }
     }
 
@@ -1135,6 +1214,11 @@ void MainWindow::on_actionPortuguese_triggered()
     loadTranslation("pt_BR");
 }
 
+void MainWindow::on_actionSpanish_triggered()
+{
+    loadTranslation("es");
+}
+
 void MainWindow::on_actionPlay_toggled(const bool checked)
 {
     if (!m_currentTab) {
@@ -1172,9 +1256,10 @@ void MainWindow::populateMenu(QSpacerItem *spacer, const QStringList &names, QLa
 void MainWindow::populateLeftMenu()
 {
     m_ui->tabElements->setCurrentIndex(0);
-    populateMenu(m_ui->verticalSpacer_InOut, {"InputVcc", "InputGnd", "InputButton", "InputSwitch", "InputRotary", "Clock", "Led", "Display7", "Display14", "Buzzer"}, m_ui->scrollAreaWidgetContents_InOut->layout());
-    populateMenu(m_ui->verticalSpacer_Gates, {"And", "Or", "Not", "Nand", "Nor", "Xor", "Xnor", "Mux", "Demux", "Node"}, m_ui->scrollAreaWidgetContents_Gates->layout());
-    populateMenu(m_ui->verticalSpacer_Memory, {"DLatch", "DFlipFlop", "JKFlipFlop", "SRFlipFlop", "TFlipFlop"}, m_ui->scrollAreaWidgetContents_Memory->layout());
+    populateMenu(m_ui->verticalSpacer_InOut, {"InputVcc", "InputGnd", "InputButton", "InputSwitch", "InputRotary", "Clock", "Led", "Display7", "Display14", "Buzzer", "AudioBox"}, m_ui->scrollAreaWidgetContents_InOut->layout());
+    populateMenu(m_ui->verticalSpacer_Gates, {"And", "Or", "Not", "Nand", "Nor", "Xor", "Xnor", "Node"}, m_ui->scrollAreaWidgetContents_Gates->layout());
+    populateMenu(m_ui->verticalSpacer_Combinational, {"TruthTable", "Mux", "Demux"}, m_ui->scrollAreaWidgetContents_Combinational->layout());
+    populateMenu(m_ui->verticalSpacer_Memory, {"DLatch", "SRLatch", "DFlipFlop", "JKFlipFlop", "TFlipFlop"}, m_ui->scrollAreaWidgetContents_Memory->layout());
     populateMenu(m_ui->verticalSpacer_Misc, {"Text", "Line"}, m_ui->scrollAreaWidgetContents_Misc->layout());
 }
 
@@ -1190,7 +1275,7 @@ void MainWindow::on_actionWaveform_triggered()
         return;
     }
 
-    qCDebug(zero) << tr("BD fileName: ") << m_currentTab->dolphinFileName();
+    qCDebug(zero) << "BD fileName: " << m_currentTab->dolphinFileName();
     auto *bewavedDolphin = new BewavedDolphin(m_currentTab->scene(), true, this);
     bewavedDolphin->createWaveform(m_currentTab->dolphinFileName());
     bewavedDolphin->show();
@@ -1292,7 +1377,9 @@ bool MainWindow::event(QEvent *event)
     }
 
     case QEvent::WindowDeactivate: {
-        on_actionPlay_toggled(false);
+        if (!m_ui->actionBackground_Simulation->isChecked()) {
+            on_actionPlay_toggled(false);
+        }
         break;
     }
 
