@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "scene.h"
@@ -55,22 +55,25 @@ void Scene::checkUpdateRequest()
 
 void Scene::drawBackground(QPainter *painter, const QRectF &rect)
 {
+    // don't draw background if zoomed out
+    if (view() and view()->transform().m11() < 0.3) {
+        return;
+    }
+
     painter->setRenderHint(QPainter::Antialiasing, true);
     QGraphicsScene::drawBackground(painter, rect);
 
     const int gridSize = GlobalProperties::gridSize;
     const int left = static_cast<int>(rect.left()) - (static_cast<int>(rect.left()) % gridSize);
     const int top = static_cast<int>(rect.top()) - (static_cast<int>(rect.top()) % gridSize);
-    QVector<QPoint> points;
+    painter->setPen(m_dots);
 
+    // TODO: replace this with a QPixmap for better performance
     for (int x = left; x < rect.right(); x += gridSize) {
         for (int y = top; y < rect.bottom(); y += gridSize) {
-            points.append(QPoint(x, y));
+            painter->drawPoint(x, y);
         }
     }
-
-    painter->setPen(m_dots);
-    painter->drawPoints(points.data(), points.size());
 }
 
 void Scene::setDots(const QPen &dots)
@@ -172,13 +175,13 @@ QGraphicsItem *Scene::itemAt(const QPointF pos)
     auto items_ = items(pos);
     items_.append(itemsAt(pos));
 
-    for (auto *item : qAsConst(items_)) {
+    for (auto *item : std::as_const(items_)) {
         if (item->type() == QNEPort::Type) {
             return item;
         }
     }
 
-    for (auto *item : qAsConst(items_)) {
+    for (auto *item : std::as_const(items_)) {
         if (item->type() > QGraphicsItem::UserType) {
             return item;
         }
@@ -203,13 +206,11 @@ void Scene::resizeScene()
 {
     setSceneRect(itemsBoundingRect());
 
-    auto *item = itemAt(m_mousePos);
-
-    if (item && (m_timer.elapsed() > 100) && m_draggingElement) {
-        // FIXME: sometimes this goes into a infinite loop and crashes
-        item->ensureVisible();
-        m_timer.restart();
-    }
+    // if (auto *item = itemAt(m_mousePos); item && (m_timer.elapsed() > 100) && m_draggingElement) {
+    //     // FIXME: sometimes this goes into a infinite loop and crashes
+    //     item->ensureVisible();
+    //     m_timer.restart();
+    // }
 }
 
 QNEConnection *Scene::editedConnection() const
@@ -310,9 +311,189 @@ void Scene::detachConnection(QNEInputPort *endPort)
     }
 }
 
+void Scene::prevMainPropShortcut()
+{
+    for (auto *element : selectedElements()) {
+        switch (element->elementType()) {
+        // Logic Elements
+        case ElementType::And:
+        case ElementType::Or:
+        case ElementType::Nand:
+        case ElementType::Nor:
+        case ElementType::Xor:
+        case ElementType::Xnor:
+        // Output and truthtable
+        case ElementType::Led:
+        case ElementType::TruthTable:
+            if (element->inputSize() > element->minInputSize())
+                receiveCommand(new ChangeInputSizeCommand(QList<GraphicElement *>{element},
+                                                          element->inputSize() - 1, this));
+            break;
+
+        // Input ports
+        case ElementType::InputRotary:
+            if (element->outputSize() > element->minOutputSize())
+                receiveCommand(new ChangeOutputSizeCommand(QList<GraphicElement *>{element},
+                                                           element->outputSize() - 1, this));
+            break;
+
+        case ElementType::Clock:
+            if (element->hasFrequency())
+                element->setFrequency(element->frequency() - 0.5);
+            break;
+
+        case ElementType::Buzzer:
+            if (element->hasAudio())
+                element->setAudio(element->previousAudio());
+            break;
+
+        case ElementType::Display14:
+        case ElementType::Display7:
+            if (element->hasColors())
+                element->setColor(element->previousColor());
+            break;
+
+        default: // Not implemented
+            break;
+        }
+
+        element->setSelected(false);
+        element->setSelected(true);
+    }
+}
+
+void Scene::nextMainPropShortcut()
+{
+    for (auto *element : selectedElements()) {
+        switch (element->elementType()) {
+        // Logic Elements
+        case ElementType::And:
+        case ElementType::Or:
+        case ElementType::Nand:
+        case ElementType::Nor:
+        case ElementType::Xor:
+        case ElementType::Xnor:
+        // Output and truthtable
+        case ElementType::Led:
+        case ElementType::TruthTable:
+            if (element->inputSize() < element->maxInputSize())
+                receiveCommand(new ChangeInputSizeCommand(QList<GraphicElement *>{element},
+                                                          element->inputSize() + 1, this));
+            break;
+
+        // Input ports
+        case ElementType::InputRotary:
+            if (element->outputSize() < element->maxOutputSize())
+                receiveCommand(new ChangeOutputSizeCommand(QList<GraphicElement *>{element},
+                                                           element->outputSize() + 1, this));
+            break;
+
+        case ElementType::Clock:
+            if (element->hasFrequency())
+                element->setFrequency(element->frequency() + 0.5);
+            break;
+
+        case ElementType::Buzzer:
+            if (element->hasAudio())
+                element->setAudio(element->nextAudio());
+            break;
+
+        case ElementType::Display14:
+        case ElementType::Display7:
+            if (element->hasColors())
+                element->setColor(element->nextColor());
+            break;
+
+        default: // Not implemented
+            break;
+        }
+
+        element->setSelected(false);
+        element->setSelected(true);
+    }
+}
+
+void Scene::prevSecndPropShortcut()
+{
+    for (auto *element : selectedElements()) {
+        switch (element->elementType()) {
+        case ElementType::TruthTable:
+            if (element->outputSize() > element->minOutputSize())
+                receiveCommand(new ChangeOutputSizeCommand(QList<GraphicElement *>{element},
+                                                           element->outputSize() - 1, this));
+            break;
+
+        case ElementType::Led:
+            if (element->hasColors())
+                element->setColor(element->previousColor());
+            break;
+
+        default:
+            break;
+        }
+
+        element->setSelected(false);
+        element->setSelected(true);
+    }
+}
+
+void Scene::nextSecndPropShortcut()
+{
+    for (auto *element : selectedElements()) {
+        switch (element->elementType()) {
+        case ElementType::TruthTable:
+            if (element->outputSize() < element->maxOutputSize())
+                receiveCommand(new ChangeOutputSizeCommand(QList<GraphicElement *>{element},
+                                                           element->outputSize() + 1, this));
+            break;
+
+        case ElementType::Led:
+            if (element->hasColors())
+                element->setColor(element->nextColor());
+            break;
+
+        default:
+            break;
+        }
+
+        element->setSelected(false);
+        element->setSelected(true);
+    }
+}
+
+void Scene::nextElm()
+{
+    for (auto *element : selectedElements()) {
+        const QPointF elmPosition = element->scenePos();
+        auto nextType = Enums::nextElmType(element->elementType());
+
+        if (nextType == ElementType::Unknown) { continue; }
+
+        receiveCommand(new MorphCommand(QList<GraphicElement *>{element},
+                       Enums::nextElmType(element->elementType()), this));
+
+        itemAt(elmPosition)->setSelected(true);
+    }
+}
+
+void Scene::prevElm()
+{
+    for (auto *element : selectedElements()) {
+        const QPointF elmPosition = element->scenePos();
+        auto prevType = Enums::prevElmType(element->elementType());
+
+        if (prevType == ElementType::Unknown) { continue; }
+
+        receiveCommand(new MorphCommand(QList<GraphicElement *>{element},
+                                        Enums::prevElmType(element->elementType()), this));
+
+        itemAt(elmPosition)->setSelected(true);
+    }
+}
+
 void Scene::updateTheme()
 {
-    qCDebug(zero) << tr("Updating theme.");
+    qCDebug(zero) << "Updating theme.";
     const ThemeAttributes theme = ThemeManager::attributes();
     setBackgroundBrush(theme.m_sceneBgBrush);
     setDots(QPen(theme.m_sceneBgDots));
@@ -327,7 +508,7 @@ void Scene::updateTheme()
         conn->updateTheme();
     }
 
-    qCDebug(zero) << tr("Finished updating theme.");
+    qCDebug(zero) << "Finished updating theme.";
 }
 
 QList<QGraphicsItem *> Scene::items(Qt::SortOrder order) const
@@ -392,7 +573,7 @@ void Scene::showWires(const bool checked)
 
 void Scene::cloneDrag(const QPointF mousePos)
 {
-    qCDebug(zero) << tr("Ctrl + Drag action triggered.");
+    qCDebug(zero) << "Ctrl + Drag action triggered.";
     const auto selectedElements_ = selectedElements();
 
     if (selectedElements_.isEmpty()) {
@@ -433,12 +614,12 @@ void Scene::cloneDrag(const QPointF mousePos)
 
     QByteArray itemData;
     QDataStream stream(&itemData, QIODevice::WriteOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    Serialization::writeHeaderPanda(stream);
     stream << mousePos;
     copy(selectedItems(), stream);
 
     auto *mimeData = new QMimeData();
-    mimeData->setData("wpanda/ctrlDragData", itemData);
+    mimeData->setData("application/x-wiredpanda-cloneDrag", itemData);
 
     auto *drag = new QDrag(this);
     drag->setMimeData(mimeData);
@@ -456,7 +637,7 @@ void Scene::copy(const QList<QGraphicsItem *> &items, QDataStream &stream)
     for (auto *item : items) {
         if (item->type() == GraphicElement::Type) {
             center += item->pos();
-            itemsQuantity++;
+            ++itemsQuantity;
         }
     }
 
@@ -601,11 +782,11 @@ void Scene::copyAction()
 
     QByteArray itemData;
     QDataStream stream(&itemData, QIODevice::WriteOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    Serialization::writeHeaderPanda(stream);
     copy(selectedItems(), stream);
 
     auto *mimeData = new QMimeData();
-    mimeData->setData("wpanda/copydata", itemData);
+    mimeData->setData("application/x-wiredpanda-clipboard", itemData);
 
     QApplication::clipboard()->setMimeData(mimeData);
 }
@@ -619,11 +800,11 @@ void Scene::cutAction()
 
     QByteArray itemData;
     QDataStream stream(&itemData, QIODevice::WriteOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    Serialization::writeHeaderPanda(stream);
     cut(selectedItems(), stream);
 
     auto *mimeData = new QMimeData();
-    mimeData->setData("wpanda/copydata", itemData);
+    mimeData->setData("application/x-wiredpanda-clipboard", itemData);
 
     QApplication::clipboard()->setMimeData(mimeData);
 }
@@ -632,23 +813,31 @@ void Scene::pasteAction()
 {
     const auto *mimeData = QApplication::clipboard()->mimeData();
 
+    QByteArray itemData;
+
     if (mimeData->hasFormat("wpanda/copydata")) {
-        QByteArray itemData = mimeData->data("wpanda/copydata");
+        itemData = mimeData->data("wpanda/copydata");
+    }
+
+    if (mimeData->hasFormat("application/x-wiredpanda-clipboard")) {
+        itemData = mimeData->data("application/x-wiredpanda-clipboard");
+    }
+
+    if (!itemData.isEmpty()) {
         QDataStream stream(&itemData, QIODevice::ReadOnly);
-        stream.setVersion(QDataStream::Qt_5_12);
-        paste(stream);
+        QVersionNumber version = Serialization::readHeaderPanda(stream);
+        paste(stream, version);
     }
 }
 
-void Scene::paste(QDataStream &stream)
+void Scene::paste(QDataStream &stream, QVersionNumber version)
 {
     clearSelection();
 
-    QPointF ctr; stream >> ctr;
+    QPointF center; stream >> center;
 
-    const QVersionNumber version = GlobalProperties::version;
     const auto itemList = Serialization::deserialize(stream, {}, version);
-    const QPointF offset = m_mousePos - ctr - QPointF(32.0, 32.0);
+    const QPointF offset = m_mousePos - center - QPointF(32.0, 32.0);
 
     receiveCommand(new AddItemsCommand(itemList, this));
 
@@ -737,7 +926,10 @@ void Scene::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
 {
     const auto formats = event->mimeData()->formats();
 
-    if (formats.contains("wpanda/x-dnditemdata") || formats.contains("wpanda/ctrlDragData")) {
+    if (formats.contains("wpanda/x-dnditemdata")
+        || formats.contains("wpanda/ctrlDragData")
+        || formats.contains("application/x-wiredpanda-dragdrop")
+        || formats.contains("application/x-wiredpanda-cloneDrag")) {
         event->accept();
         return;
     }
@@ -749,7 +941,10 @@ void Scene::dragMoveEvent(QGraphicsSceneDragDropEvent *event)
 {
     const auto formats = event->mimeData()->formats();
 
-    if (formats.contains("wpanda/x-dnditemdata") || formats.contains("wpanda/ctrlDragData")) {
+    if (formats.contains("wpanda/x-dnditemdata")
+        || formats.contains("wpanda/ctrlDragData")
+        || formats.contains("application/x-wiredpanda-dragdrop")
+        || formats.contains("application/x-wiredpanda-cloneDrag")) {
         event->accept();
         return;
     }
@@ -759,20 +954,30 @@ void Scene::dragMoveEvent(QGraphicsSceneDragDropEvent *event)
 
 void Scene::dropEvent(QGraphicsSceneDragDropEvent *event)
 {
-    if (event->mimeData()->hasFormat("wpanda/x-dnditemdata")) {
-        QByteArray itemData = event->mimeData()->data("wpanda/x-dnditemdata");
+    if (event->mimeData()->hasFormat("wpanda/x-dnditemdata")
+        || event->mimeData()->hasFormat("application/x-wiredpanda-dragdrop")) {
+        QByteArray itemData;
+
+        if (event->mimeData()->hasFormat("wpanda/x-dnditemdata")) {
+            itemData = event->mimeData()->data("wpanda/x-dnditemdata");
+        }
+
+        if (event->mimeData()->hasFormat("application/x-wiredpanda-dragdrop")) {
+            itemData = event->mimeData()->data("application/x-wiredpanda-dragdrop");
+        }
+
         QDataStream stream(&itemData, QIODevice::ReadOnly);
-        stream.setVersion(QDataStream::Qt_5_12);
+        Serialization::readHeaderPanda(stream);
 
         QPoint offset;      stream >> offset;
         ElementType type;   stream >> type;
         QString icFileName; stream >> icFileName;
 
         QPointF pos = event->scenePos() - offset;
-        qCDebug(zero) << type << tr(" at position: ") << pos.x() << tr(", ") << pos.y() << tr(", label: ") << icFileName;
+        qCDebug(zero) << type << " at position: " << pos.x() << ", " << pos.y() << ", label: " << icFileName;
 
         auto *element = ElementFactory::buildElement(type);
-        qCDebug(zero) << tr("Valid element.");
+        qCDebug(zero) << "Valid element.";
 
         if (element->elementType() == ElementType::IC) {
             if (auto *ic = qobject_cast<IC *>(element)) {
@@ -780,29 +985,38 @@ void Scene::dropEvent(QGraphicsSceneDragDropEvent *event)
             }
         }
 
-        qCDebug(zero) << tr("Adding the element to the scene.");
+        qCDebug(zero) << "Adding the element to the scene.";
         receiveCommand(new AddItemsCommand({element}, this));
 
-        qCDebug(zero) << tr("Cleaning the selection.");
+        qCDebug(zero) << "Cleaning the selection.";
         clearSelection();
 
-        qCDebug(zero) << tr("Setting created element as selected.");
+        qCDebug(zero) << "Setting created element as selected.";
         element->setSelected(true);
 
-        qCDebug(zero) << tr("Adjusting the position of the element.");
+        qCDebug(zero) << "Adjusting the position of the element.";
         element->setPos(pos);
     }
 
-    if (event->mimeData()->hasFormat("wpanda/ctrlDragData")) {
-        QByteArray itemData = event->mimeData()->data("wpanda/ctrlDragData");
+    if (event->mimeData()->hasFormat("wpanda/ctrlDragData")
+        || event->mimeData()->hasFormat("application/x-wiredpanda-cloneDrag")) {
+        QByteArray itemData;
+
+        if (event->mimeData()->hasFormat("wpanda/ctrlDragData")) {
+            itemData = event->mimeData()->data("wpanda/ctrlDragData");
+        }
+
+        if (event->mimeData()->hasFormat("application/x-wiredpanda-cloneDrag")) {
+            itemData = event->mimeData()->data("application/x-wiredpanda-cloneDrag");
+        }
+
         QDataStream stream(&itemData, QIODevice::ReadOnly);
-        stream.setVersion(QDataStream::Qt_5_12);
+        QVersionNumber version = Serialization::readHeaderPanda(stream);
 
         QPointF offset; stream >> offset;
         QPointF ctr;    stream >> ctr;
         offset = event->scenePos() - offset;
 
-        const QVersionNumber version = GlobalProperties::version;
         const auto itemList = Serialization::deserialize(stream, {}, version);
 
         receiveCommand(new AddItemsCommand(itemList, this));
@@ -895,7 +1109,7 @@ void Scene::mousePressEvent(QGraphicsSceneMouseEvent *event)
             m_movedElements.clear();
             m_oldPositions.clear();
 
-            for (auto *element : qAsConst(selectedElements_)) {
+            for (auto *element : std::as_const(selectedElements_)) {
                 m_movedElements.append(element);
                 m_oldPositions.append(element->pos());
             }
@@ -1016,27 +1230,18 @@ void Scene::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
     QGraphicsScene::mouseDoubleClickEvent(event);
 }
 
-void Scene::addItem(QGraphicsItem *item)
-{
-    if (auto *element = dynamic_cast<Buzzer *>(item); element && element->label().isEmpty()) {
-        element->setLabel(element->objectName() + "_" + QString::number(++m_buzzerLabelNumber));
-    }
-
-    QGraphicsScene::addItem(item);
-}
-
 void Scene::addItem(QMimeData *mimeData)
 {
     QByteArray itemData = mimeData->data("wpanda/x-dnditemdata");
     QDataStream stream(&itemData, QIODevice::ReadOnly);
-    stream.setVersion(QDataStream::Qt_5_12);
+    Serialization::readHeaderPanda(stream);
 
     QPoint offset;      stream >> offset;
     ElementType type;   stream >> type;
     QString icFileName; stream >> icFileName;
 
     auto *element = ElementFactory::buildElement(type);
-    qCDebug(zero) << tr("Valid element.");
+    qCDebug(zero) << "Valid element.";
 
     if (element->elementType() == ElementType::IC) {
         if (auto *ic = qobject_cast<IC *>(element)) {
@@ -1044,13 +1249,13 @@ void Scene::addItem(QMimeData *mimeData)
         }
     }
 
-    qCDebug(zero) << tr("Adding the element to the scene.");
+    qCDebug(zero) << "Adding the element to the scene.";
     receiveCommand(new AddItemsCommand({element}, this));
 
-    qCDebug(zero) << tr("Cleaning the selection.");
+    qCDebug(zero) << "Cleaning the selection.";
     clearSelection();
 
-    qCDebug(zero) << tr("Setting created element as selected.");
+    qCDebug(zero) << "Setting created element as selected.";
     element->setSelected(true);
 
     mimeData->deleteLater();
