@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
@@ -10,9 +10,10 @@
 #include <QGraphicsScene>
 #include <QMimeData>
 #include <QUndoCommand>
+#include <QVersionNumber>
 
-class GraphicsView;
 class GraphicElement;
+class GraphicsView;
 class QNEConnection;
 
 class Scene : public QGraphicsScene
@@ -37,7 +38,6 @@ public:
     const QVector<GraphicElement *> elements() const;
     const QVector<GraphicElement *> elements(const QRectF &rect) const;
     const QVector<GraphicElement *> visibleElements() const;
-    void addItem(QGraphicsItem *item);
     void addItem(QMimeData *mimeData);
     void copyAction();
     void cutAction();
@@ -45,7 +45,13 @@ public:
     void flipHorizontally();
     void flipVertically();
     void mute(const bool mute = true);
+    void nextElm();
+    void nextMainPropShortcut();
+    void nextSecndPropShortcut();
     void pasteAction();
+    void prevElm();
+    void prevMainPropShortcut();
+    void prevSecndPropShortcut();
     void receiveCommand(QUndoCommand *cmd);
     void rotateLeft();
     void rotateRight();
@@ -89,7 +95,7 @@ private:
     void drawBackground(QPainter *painter, const QRectF &rect) override;
     void handleHoverPort();
     void makeConnection(QNEConnection *connection);
-    void paste(QDataStream &stream);
+    void paste(QDataStream &stream, QVersionNumber version);
     void releaseHoverPort();
     void resizeScene();
     void rotate(const int angle);
@@ -117,7 +123,6 @@ private:
     bool m_markingSelectionBox = false;
     bool m_showGates = true;
     bool m_showWires = true;
-    int m_buzzerLabelNumber = 0;
     int m_editedConnectionId = 0;
     int m_hoverPortElmId = 0;
     int m_hoverPortNumber = 0;
