@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
@@ -17,5 +17,6 @@ public:
     static void setValue(const QString &key, const QVariant &value);
 
 private:
-    inline static QSettings *const settings = new QSettings(QSettings::IniFormat, QSettings::UserScope, "GIBIS-UNIFESP", "WiRedPanda");
+    static QSettings *settingsInstance();
+    static QSettings *settings;
 };
