@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "simulationblocker.h"
@@ -9,7 +9,7 @@
 SimulationBlocker::SimulationBlocker(Simulation *simulation)
     : m_simulation(simulation)
 {
-    qCDebug(zero) << QObject::tr("Stopping.");
+    qCDebug(zero) << "Stopping.";
 
     if (m_simulation->isRunning()) {
         m_restart = true;
@@ -19,7 +19,7 @@ SimulationBlocker::SimulationBlocker(Simulation *simulation)
 
 SimulationBlocker::~SimulationBlocker()
 {
-    qCDebug(zero) << QObject::tr("Releasing.");
+    qCDebug(zero) << "Releasing.";
 
     if (m_restart) {
         m_simulation->start();
