@@ -1,4 +1,4 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
@@ -25,7 +25,7 @@ public:
     QString dolphinFileName();
     Scene *scene();
     Simulation *simulation();
-    void load(QDataStream &stream);
+    void load(QDataStream &stream, QVersionNumber number);
     void load(const QString &fileName);
     void save(QDataStream &stream);
     void save(const QString &fileName);
