@@ -1,26 +1,30 @@
-// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
+// Copyright 2015 - 2025, GIBIS-UNIFESP and the wiRedPanda contributors
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "testwaveform.h"
 
 #include "bewaveddolphin.h"
+#include "serialization.h"
 #include "workspace.h"
 
 #include <QTemporaryFile>
 #include <QTest>
 
+#define QUOTE(string) _QUOTE(string)
+#define _QUOTE(string) #string
+
 void TestWaveForm::testDisplay4Bits()
 {
     WorkSpace workspace;
 
-    const QDir examplesDir(QString(CURRENTDIR) + "/../examples/");
+    const QDir examplesDir(QString(QUOTE(CURRENTDIR)) + "/../examples/");
     const QString fileName = examplesDir.absoluteFilePath("display-4bits.panda");
 
     QFile pandaFile(fileName);
     QVERIFY(pandaFile.open(QIODevice::ReadOnly));
     QDataStream stream(&pandaFile);
-    stream.setVersion(QDataStream::Qt_5_12);
-    workspace.load(stream);
+    QVersionNumber version = Serialization::readHeaderPanda(stream);
+    workspace.load(stream, version);
 
     BewavedDolphin bewavedDolphin(workspace.scene(), false);
     bewavedDolphin.createWaveform("");
